"""Handlers for MCP Text Editor."""

from .append_text_file_contents import AppendTextFileContentsHandler
from .create_text_file import CreateTextFileHandler
from .delete_text_file_contents import DeleteTextFileContentsHandler
from .get_text_file_contents import GetTextFileContentsHandler
from .insert_text_file_contents import InsertTextFileContentsHandler
from .patch_text_file_contents import PatchTextFileContentsHandler

__all__ = [
    "AppendTextFileContentsHandler",
    "CreateTextFileHandler",
    "DeleteTextFileContentsHandler",
    "GetTextFileContentsHandler",
    "InsertTextFileContentsHandler",
    "PatchTextFileContentsHandler",
]



"""Handler for deleting content from text files."""

import json
import logging
import os
import traceback
from typing import Any, Dict, Sequence

from mcp.types import TextContent, Tool

from ..models import DeleteTextFileContentsRequest, FileRange
from .base import BaseHandler

logger = logging.getLogger("mcp-text-editor")


class DeleteTextFileContentsHandler(BaseHandler):
    """Handler for deleting content from a text file."""

    name = "delete_text_file_contents"
    description = "Delete specified content ranges from a text file. The file must exist. File paths must be absolute. You need to provide the file_hash comes from get_text_file_contents."

    def get_tool_description(self) -> Tool:
        """Get the tool description."""
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "file_path": {
                        "type": "string",
                        "description": "Path to the text file. File path must be absolute.",
                    },
                    "file_hash": {
                        "type": "string",
                        "description": "Hash of the file contents for concurrency control. it should be matched with the file_hash when get_text_file_contents is called.",
                    },
                    "ranges": {
                        "type": "array",
                        "description": "List of line ranges to delete",
                        "items": {
                            "type": "object",
                            "properties": {
                                "start": {
                                    "type": "integer",
                                    "description": "Starting line number (1-based)",
                                },
                                "end": {
                                    "type": ["integer", "null"],
                                    "description": "Ending line number (null for end of file)",
                                },
                                "range_hash": {
                                    "type": "string",
                                    "description": "Hash of the content being deleted. it should be matched with the range_hash when get_text_file_contents is called with the same range.",
                                },
                            },
                            "required": ["start", "range_hash"],
                        },
                    },
                    "encoding": {
                        "type": "string",
                        "description": "Text encoding (default: 'utf-8')",
                        "default": "utf-8",
                    },
                },
                "required": ["file_path", "file_hash", "ranges"],
            },
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        """Execute the tool with given arguments."""
        try:
            # Input validation
            if "file_path" not in arguments:
                raise RuntimeError("Missing required argument: file_path")
            if "file_hash" not in arguments:
                raise RuntimeError("Missing required argument: file_hash")
            if "ranges" not in arguments:
                raise RuntimeError("Missing required argument: ranges")

            file_path = arguments["file_path"]
            if not os.path.isabs(file_path):
                raise RuntimeError(f"File path must be absolute: {file_path}")

            # Check if file exists
            if not os.path.exists(file_path):
                raise RuntimeError(f"File does not exist: {file_path}")

            encoding = arguments.get("encoding", "utf-8")

            # Create file ranges for deletion
            ranges = [
                FileRange(
                    start=r["start"], end=r.get("end"), range_hash=r["range_hash"]
                )
                for r in arguments["ranges"]
            ]

            # Create delete request
            request = DeleteTextFileContentsRequest(
                file_path=file_path,
                file_hash=arguments["file_hash"],
                ranges=ranges,
                encoding=encoding,
            )

            # Execute deletion using the service
            result_dict = self.editor.service.delete_text_file_contents(request)

            # Convert EditResults to dictionaries
            serializable_result = {}
            for file_path, edit_result in result_dict.items():
                serializable_result[file_path] = edit_result.to_dict()

            return [
                TextContent(type="text", text=json.dumps(serializable_result, indent=2))
            ]

        except Exception as e:
            logger.error(f"Error processing request: {str(e)}")
            logger.error(traceback.format_exc())
            raise RuntimeError(f"Error processing request: {str(e)}") from e



"""Test module for patch_text_file."""

import os

import pytest

from mcp_text_editor.handlers.patch_text_file_contents import (
    PatchTextFileContentsHandler,
)
from mcp_text_editor.text_editor import TextEditor


@pytest.mark.asyncio
async def test_patch_text_file_middle(tmp_path):
    """Test patching text in the middle of the file."""
    # Create a test file
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\nline3\nline4\nline5\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Get file hash and range hash
    file_info = await editor.read_multiple_ranges(
        [{"file_path": str(file_path), "ranges": [{"start": 2, "end": 3}]}]
    )

    # Extract file and range hashes
    file_content = file_info[str(file_path)]
    file_hash = file_content["file_hash"]
    range_hash = file_content["ranges"][0]["range_hash"]

    # Patch the file
    new_content = "new line2\nnew line3\n"
    patch = {
        "start": 2,
        "end": 3,  # changed from 4 to 3 since we only want to replace lines 2-3
        "contents": new_content,
        "range_hash": range_hash,
    }
    result = await editor.edit_file_contents(
        str(file_path),
        file_hash,
        [patch],
    )

    # Verify the patch was successful
    assert result["result"] == "ok"
    with open(file_path, "r", encoding="utf-8") as f:
        updated_content = f.read()
    assert updated_content == "line1\nnew line2\nnew line3\nline4\nline5\n"


@pytest.mark.asyncio
async def test_patch_text_file_empty_content(tmp_path):
    """Test patching with empty content suggests using delete_text_file_contents."""
    # Create a test file
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\nline3\nline4\nline5\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Get file hash and range hash
    file_info = await editor.read_multiple_ranges(
        [{"file_path": str(file_path), "ranges": [{"start": 2, "end": 3}]}]
    )

    # Extract file and range hashes
    file_content = file_info[str(file_path)]
    file_hash = file_content["file_hash"]
    range_hash = file_content["ranges"][0]["range_hash"]

    # Patch the file with empty content
    patch = {
        "start": 2,
        "end": 3,
        "contents": "",
        "range_hash": range_hash,
    }
    result = await editor.edit_file_contents(
        str(file_path),
        file_hash,
        [patch],
    )

    # Verify that the operation suggests using delete_text_file_contents
    assert result["result"] == "ok"
    assert result["file_hash"] == file_hash
    assert "delete_text_file_contents" in result["hint"]
    assert result["suggestion"] == "delete"

    # Verify file content remains unchanged
    with open(file_path, "r", encoding="utf-8") as f:
        updated_content = f.read()
    assert updated_content == content


@pytest.mark.asyncio
async def test_patch_text_file_errors(tmp_path):
    """Test error handling when patching a file."""
    editor = TextEditor()
    handler = PatchTextFileContentsHandler(editor)
    file_path = os.path.join(tmp_path, "test.txt")
    non_existent_file = os.path.join(tmp_path, "non_existent.txt")
    relative_path = "test.txt"

    # Test missing file_path
    with pytest.raises(RuntimeError, match="Missing required argument: file_path"):
        await handler.run_tool({"patches": [], "file_hash": "hash"})

    # Test missing file_hash
    with pytest.raises(RuntimeError, match="Missing required argument: file_hash"):
        await handler.run_tool({"file_path": file_path, "patches": []})

    # Test missing patches
    with pytest.raises(RuntimeError, match="Missing required argument: patches"):
        await handler.run_tool({"file_path": file_path, "file_hash": "hash"})

    # Test non-absolute file path
    with pytest.raises(RuntimeError, match="File path must be absolute"):
        await handler.run_tool(
            {"file_path": relative_path, "file_hash": "hash", "patches": []}
        )

    # Test non-existent file
    with pytest.raises(RuntimeError, match="File does not exist"):
        await handler.run_tool(
            {"file_path": non_existent_file, "file_hash": "hash", "patches": []}
        )


@pytest.mark.asyncio
async def test_patch_text_file_unexpected_error(tmp_path, mocker):
    """Test handling of unexpected errors during patching."""
    editor = TextEditor()
    handler = PatchTextFileContentsHandler(editor)
    file_path = os.path.join(tmp_path, "test.txt")

    # Create a test file
    with open(file_path, "w", encoding="utf-8") as f:
        f.write("test content\n")

    # Mock edit_file_contents to raise an unexpected error
    async def mock_edit_file_contents(*args, **kwargs):
        raise Exception("Unexpected test error")

    # Patch the editor's method using mocker
    mocker.patch.object(editor, "edit_file_contents", mock_edit_file_contents)

    # Try to patch the file with the mocked error
    with pytest.raises(
        RuntimeError, match="Error processing request: Unexpected test error"
    ):
        await handler.run_tool(
            {
                "file_path": file_path,
                "file_hash": "dummy_hash",
                "patches": [
                    {
                        "start": 1,
                        "contents": "new content\n",
                        "range_hash": "dummy_hash",
                    }
                ],
            }
        )


@pytest.mark.asyncio
async def test_patch_text_file_overlapping(tmp_path):
    """Test patching text with overlapping ranges should fail."""
    # Create a test file
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\nline3\nline4\nline5\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Get file hash and range hash for first patch
    file_info = await editor.read_multiple_ranges(
        [{"file_path": str(file_path), "ranges": [{"start": 2, "end": 3}]}]
    )

    # Extract hashes for first patch
    file_content = file_info[str(file_path)]
    file_hash = file_content["file_hash"]
    range_hash_1 = file_content["ranges"][0]["range_hash"]

    # Get range hash for second patch
    file_info = await editor.read_multiple_ranges(
        [{"file_path": str(file_path), "ranges": [{"start": 3, "end": 4}]}]
    )
    range_hash_2 = file_info[str(file_path)]["ranges"][0]["range_hash"]

    # Create overlapping patches
    patches = [
        {
            "start": 2,
            "end": 3,
            "contents": "new line2\nnew line3\n",
            "range_hash": range_hash_1,
        },
        {
            "start": 3,
            "end": 4,
            "contents": "another new line3\nnew line4\n",
            "range_hash": range_hash_2,
        },
    ]

    # Try to apply overlapping patches
    result = await editor.edit_file_contents(
        str(file_path),
        file_hash,
        patches,
    )

    # Verify that the operation failed due to overlapping patches
    assert result["result"] == "error"


@pytest.mark.asyncio
async def test_patch_text_file_new_file_hint(tmp_path):
    """Test patching a new file suggests using append_text_file_contents."""
    file_path = os.path.join(tmp_path, "new.txt")
    editor = TextEditor()

    # Try to patch a new file
    patch = {
        "start": 1,
        "contents": "new content\n",
        "range_hash": "",  # Empty hash for new file
    }
    result = await editor.edit_file_contents(
        str(file_path),
        "",  # Empty hash for new file
        [patch],
    )

    # Verify that the operation suggests using append_text_file_contents
    assert result["result"] == "ok"
    assert result["suggestion"] == "append"
    assert "append_text_file_contents" in result["hint"]


@pytest.mark.asyncio
async def test_patch_text_file_append_hint(tmp_path):
    """Test patching beyond file end suggests using append_text_file_contents."""
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Get file hash
    file_info = await editor.read_multiple_ranges(
        [{"file_path": str(file_path), "ranges": [{"start": 1, "end": 2}]}]
    )
    file_hash = file_info[str(file_path)]["file_hash"]

    # Try to patch beyond file end
    patch = {
        "start": 5,  # Beyond file end
        "contents": "new line\n",
        "range_hash": "",  # Empty hash for insertion
    }
    result = await editor.edit_file_contents(
        str(file_path),
        file_hash,
        [patch],
    )

    # Verify the suggestion to use append_text_file_contents
    assert result["result"] == "ok"
    assert result["suggestion"] == "append"
    assert "append_text_file_contents" in result["hint"]


@pytest.mark.asyncio
async def test_patch_text_file_insert_hint(tmp_path):
    """Test patching with insertion suggests using insert_text_file_contents."""
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Get file hash
    file_info = await editor.read_multiple_ranges(
        [{"file_path": str(file_path), "ranges": [{"start": 1, "end": 2}]}]
    )
    file_hash = file_info[str(file_path)]["file_hash"]

    # Try to insert at start
    patch = {
        "start": 1,
        "contents": "new line\n",
        "range_hash": "",  # Empty hash for insertion
    }
    result = await editor.edit_file_contents(
        str(file_path),
        file_hash,
        [patch],
    )

    # Verify the suggestion to use insert_text_file_contents
    assert result["result"] == "ok"
    assert result["suggestion"] == "insert"
    assert "insert_text_file_contents" in result["hint"]


@pytest.mark.asyncio
async def test_patch_text_file_hash_mismatch_hint(tmp_path):
    """Test patching with wrong hash suggests using get_text_file_contents."""
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Try to patch with wrong file hash
    patch = {
        "start": 1,
        "contents": "new line\n",
        "range_hash": "",
    }
    result = await editor.edit_file_contents(
        str(file_path),
        "wrong_hash",
        [patch],
    )

    # Verify the suggestion to use get_text_file_contents
    assert result["result"] == "error"
    assert "get_text_file_contents" in result["hint"]



"""Core text editor functionality with file operation handling."""

import hashlib
import logging
import os
from typing import Any, Dict, List, Optional, Tuple

from .models import DeleteTextFileContentsRequest, EditPatch, FileRanges
from .service import TextEditorService

logger = logging.getLogger(__name__)


class TextEditor:
    """Handles text file operations with security checks and conflict detection."""

    def __init__(self):
        """Initialize TextEditor."""
        self._validate_environment()
        self.service = TextEditorService()

    def create_error_response(
        self,
        error_message: str,
        content_hash: Optional[str] = None,
        file_path: Optional[str] = None,
        suggestion: Optional[str] = None,
        hint: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Create a standardized error response.

        Args:
            error_message (str): The error message to include
            content_hash (Optional[str], optional): Hash of the current content if available
            file_path (Optional[str], optional): File path to use as dictionary key
            suggestion (Optional[str], optional): Suggested operation type
            hint (Optional[str], optional): Hint message for users

        Returns:
            Dict[str, Any]: Standardized error response structure
        """
        error_response = {
            "result": "error",
            "reason": error_message,
            "file_hash": content_hash,
        }

        # Add fields if provided
        if content_hash is not None:
            error_response["file_hash"] = content_hash
        if suggestion:
            error_response["suggestion"] = suggestion
        if hint:
            error_response["hint"] = hint

        if file_path:
            return {file_path: error_response}
        return error_response

    def _validate_environment(self) -> None:
        """
        Validate environment variables and setup.
        Can be extended to check for specific permissions or configurations.
        """
        # Future: Add environment validation if needed
        pass  # pragma: no cover

    def _validate_file_path(self, file_path: str | os.PathLike) -> None:
        """
        Validate if file path is allowed and secure.

        Args:
            file_path (str | os.PathLike): Path to validate

        Raises:
            ValueError: If path is not allowed or contains dangerous patterns
        """
        # Convert path to string for checking
        path_str = str(file_path)

        # Check for dangerous patterns
        if ".." in path_str:
            raise ValueError("Path traversal not allowed")

    @staticmethod
    def calculate_hash(content: str) -> str:
        """
        Calculate SHA-256 hash of content.

        Args:
            content (str): Content to hash

        Returns:
            str: Hex digest of SHA-256 hash
        """
        return hashlib.sha256(content.encode()).hexdigest()

    async def _read_file(
        self, file_path: str, encoding: str = "utf-8"
    ) -> Tuple[List[str], str, int]:
        """Read file and return lines, content, and total lines.

        Args:
            file_path (str): Path to the file to read
            encoding (str, optional): File encoding. Defaults to "utf-8"

        Returns:
            Tuple[List[str], str, int]: Lines, content, and total line count

        Raises:
            FileNotFoundError: If file not found
            UnicodeDecodeError: If file cannot be decoded with specified encoding
        """
        self._validate_file_path(file_path)
        try:
            with open(file_path, "r", encoding=encoding) as f:
                lines = f.readlines()
            file_content = "".join(lines)
            return lines, file_content, len(lines)
        except FileNotFoundError as err:
            raise FileNotFoundError(f"File not found: {file_path}") from err
        except UnicodeDecodeError as err:
            raise UnicodeDecodeError(
                encoding,
                err.object,
                err.start,
                err.end,
                f"Failed to decode file '{file_path}' with {encoding} encoding",
            ) from err

    async def read_multiple_ranges(
        self, ranges: List[Dict[str, Any]], encoding: str = "utf-8"
    ) -> Dict[str, Dict[str, Any]]:
        result: Dict[str, Dict[str, Any]] = {}

        for file_range_dict in ranges:
            file_range = FileRanges.model_validate(file_range_dict)
            file_path = file_range.file_path
            lines, file_content, total_lines = await self._read_file(
                file_path, encoding=encoding
            )
            file_hash = self.calculate_hash(file_content)
            result[file_path] = {"ranges": [], "file_hash": file_hash}

            for range_spec in file_range.ranges:
                start = max(1, range_spec.start) - 1
                end_value = range_spec.end
                end = (
                    min(total_lines, end_value)
                    if end_value is not None
                    else total_lines
                )

                if start >= total_lines:
                    empty_content = ""
                    result[file_path]["ranges"].append(
                        {
                            "content": empty_content,
                            "start": start + 1,
                            "end": start + 1,
                            "range_hash": self.calculate_hash(empty_content),
                            "total_lines": total_lines,
                            "content_size": 0,
                        }
                    )
                    continue

                selected_lines = lines[start:end]
                content = "".join(selected_lines)
                range_hash = self.calculate_hash(content)

                result[file_path]["ranges"].append(
                    {
                        "content": content,
                        "start": start + 1,
                        "end": end,
                        "range_hash": range_hash,
                        "total_lines": total_lines,
                        "content_size": len(content),
                    }
                )

        return result

    async def read_file_contents(
        self,
        file_path: str,
        start: int = 1,
        end: Optional[int] = None,
        encoding: str = "utf-8",
    ) -> Tuple[str, int, int, str, int, int]:
        lines, file_content, total_lines = await self._read_file(
            file_path, encoding=encoding
        )

        if end is not None and end < start:
            raise ValueError("End line must be greater than or equal to start line")

        start = max(1, start) - 1
        end = total_lines if end is None else min(end, total_lines)

        if start >= total_lines:
            empty_content = ""
            empty_hash = self.calculate_hash(empty_content)
            return empty_content, start, start, empty_hash, total_lines, 0
        if end < start:
            raise ValueError("End line must be greater than or equal to start line")

        selected_lines = lines[start:end]
        content = "".join(selected_lines)
        content_hash = self.calculate_hash(content)
        content_size = len(content.encode(encoding))

        return (
            content,
            start + 1,
            end,
            content_hash,
            total_lines,
            content_size,
        )

    async def edit_file_contents(
        self,
        file_path: str,
        expected_file_hash: str,
        patches: List[Dict[str, Any]],
        encoding: str = "utf-8",
    ) -> Dict[str, Any]:
        """
        Edit file contents with hash-based conflict detection and multiple patches.

        Args:
            file_path (str): Path to the file to edit
            expected_hash (str): Expected hash of the file before editing
            patches (List[Dict[str, Any]]): List of patches to apply, each containing:
                - start (int): Starting line number (1-based)
                - end (Optional[int]): Ending line number (inclusive)
                - contents (str): New content to insert (if empty string, consider using delete_text_file_contents instead)
                - range_hash (str): Expected hash of the content being replaced

        Returns:
            Dict[str, Any]: Results of the operation containing:
                - result: "ok" or "error"
                - hash: New file hash if successful, None if error
                - reason: Error message if result is "error"
                    "file_hash": None,
                }

            # Read current file content and verify hash
        """
        self._validate_file_path(file_path)
        try:
            if not os.path.exists(file_path):
                if expected_file_hash not in ["", None]:  # Allow null hash
                    return self.create_error_response(
                        "File not found and non-empty hash provided",
                        suggestion="append",
                        hint="For new files, please consider using append_text_file_contents",
                    )
                # Create parent directories if they don't exist
                parent_dir = os.path.dirname(file_path)
                if parent_dir:
                    try:
                        os.makedirs(parent_dir, exist_ok=True)
                    except OSError as e:
                        return self.create_error_response(
                            f"Failed to create directory: {str(e)}",
                            suggestion="patch",
                            hint="Please check file permissions and try again",
                        )
                # Initialize empty state for new file
                current_file_content = ""
                current_file_hash = ""
                lines: List[str] = []
                encoding = "utf-8"
            else:
                # Read current file content and verify hash
                (
                    current_file_content,
                    _,
                    _,
                    current_file_hash,
                    total_lines,
                    _,
                ) = await self.read_file_contents(file_path, encoding=encoding)

                # Treat empty file as new file
                if not current_file_content:
                    current_file_content = ""
                    current_file_hash = ""
                    lines = []
                elif current_file_content and expected_file_hash == "":
                    return self.create_error_response(
                        "Unexpected error - Cannot treat existing file as new",
                    )
                elif current_file_hash != expected_file_hash:
                    suggestion = "patch"
                    hint = "Please use get_text_file_contents tool to get the current content and hash"

                    return self.create_error_response(
                        "FileHash mismatch - Please use get_text_file_contents tool to get current content and hashes, then retry with the updated hashes.",
                        suggestion=suggestion,
                        hint=hint,
                    )
                else:
                    lines = current_file_content.splitlines(keepends=True)
                    lines = current_file_content.splitlines(keepends=True)

            # Convert patches to EditPatch objects
            patch_objects = [EditPatch.model_validate(p) for p in patches]

            # Sort patches from bottom to top to avoid line number shifts
            sorted_patches = sorted(
                patch_objects,
                key=lambda x: (
                    -(x.start),
                    -(x.end or x.start or float("inf")),
                ),
            )

            # Check for overlapping patches
            for i in range(len(sorted_patches)):
                for j in range(i + 1, len(sorted_patches)):
                    patch1 = sorted_patches[i]
                    patch2 = sorted_patches[j]
                    start1 = patch1.start
                    end1 = patch1.end or start1
                    start2 = patch2.start
                    end2 = patch2.end or start2

                    if (start1 <= end2 and end1 >= start2) or (
                        start2 <= end1 and end2 >= start1
                    ):
                        return self.create_error_response(
                            "Overlapping patches detected",
                            suggestion="patch",
                            hint="Please ensure your patches do not overlap",
                        )

            # Apply patches
            for patch in sorted_patches:
                # Get line numbers (1-based)
                start: int
                end: Optional[int]
                if isinstance(patch, EditPatch):
                    start = patch.start
                    end = patch.end
                else:
                    start = patch["start"] if "start" in patch else 1
                    end = patch["end"] if "end" in patch else start

                # Check for invalid line range
                if end is not None and end < start:
                    return {
                        "result": "error",
                        "reason": "End line must be greater than or equal to start line",
                        "file_hash": None,
                        "content": current_file_content,
                    }

                # Handle unexpected empty hash for existing file
                if (
                    os.path.exists(file_path)
                    and current_file_content
                    and expected_file_hash == ""
                ):
                    return {
                        "result": "error",
                        "reason": "File hash validation required: Empty hash provided for existing file",
                        "details": {
                            "file_path": file_path,
                            "current_file_hash": self.calculate_hash(
                                current_file_content
                            ),
                            "expected_file_hash": expected_file_hash,
                        },
                    }

                # Calculate line ranges for zero-based indexing
                start_zero = start - 1

                # Get expected hash for validation
                expected_range_hash = None
                if isinstance(patch, dict):
                    expected_range_hash = patch.get("range_hash")
                else:
                    # For EditPatch objects, use model fields
                    expected_range_hash = patch.range_hash

                # Determine operation type and validate hash requirements
                if not os.path.exists(file_path) or not current_file_content:
                    # New file or empty file - treat as insertion
                    is_insertion = True
                elif start_zero >= len(lines):
                    is_insertion = True
                else:
                    # For modification mode, check the range_hash
                    is_insertion = expected_range_hash == ""
                    if not is_insertion:
                        # Calculate end_zero for content validation
                        end_zero = (
                            len(lines) - 1
                            if end is None
                            else min(end - 1, len(lines) - 1)
                        )

                        # Hash provided - verify content
                        target_lines = lines[start_zero : end_zero + 1]
                        target_content = "".join(target_lines)
                        actual_range_hash = self.calculate_hash(target_content)

                        if actual_range_hash != expected_range_hash:
                            return {
                                "result": "error",
                                "reason": "Content range hash mismatch - Please use get_text_file_contents tool with the same start and end to get current content and hashes, then retry with the updated hashes.",
                                "suggestion": "get",
                                "hint": "Please run get_text_file_contents first to get current content and hashes",
                            }

                # Prepare new content
                if isinstance(patch, EditPatch):
                    contents = patch.contents
                else:
                    contents = patch["contents"]

                # Check if this is a deletion (empty content)
                if not contents.strip():
                    return {
                        "result": "ok",
                        "file_hash": current_file_hash,  # Return current hash since no changes made
                        "hint": "For content deletion, please consider using delete_text_file_contents instead of patch with empty content",
                        "suggestion": "delete",
                    }

                # Set suggestions for alternative tools
                suggestion_text: Optional[str] = None
                hint_text: Optional[str] = None
                if not os.path.exists(file_path) or not current_file_content:
                    suggestion_text = "append"
                    hint_text = "For new or empty files, please consider using append_text_file_contents instead"
                elif is_insertion:
                    if start_zero >= len(lines):
                        suggestion_text = "append"
                        hint_text = "For adding content at the end of file, please consider using append_text_file_contents instead"
                    else:
                        suggestion_text = "insert"
                        hint_text = "For inserting content within file, please consider using insert_text_file_contents instead"

                # Prepare the content
                new_content = contents if contents.endswith("\n") else contents + "\n"
                new_lines = new_content.splitlines(keepends=True)

                # For insertion mode, we don't need end_zero
                if is_insertion:
                    # Insert at the specified line
                    lines[start_zero:start_zero] = new_lines
                else:
                    # We already have end_zero for replacements
                    lines[start_zero : end_zero + 1] = new_lines

            # Write the final content back to file
            final_content = "".join(lines)
            with open(file_path, "w", encoding=encoding) as f:
                f.write(final_content)

            # Calculate new hash
            new_hash = self.calculate_hash(final_content)

            return {
                "result": "ok",
                "file_hash": new_hash,
                "reason": None,
                "suggestion": suggestion_text,
                "hint": hint_text,
            }

        except FileNotFoundError:
            return self.create_error_response(
                f"File not found: {file_path}",
                suggestion="append",
                hint="For new files, please use append_text_file_contents",
            )
        except (IOError, UnicodeError, PermissionError) as e:
            return self.create_error_response(
                f"Error editing file: {str(e)}",
                suggestion="patch",
                hint="Please check file permissions and try again",
            )
        except Exception as e:
            import traceback

            logger.error(f"Error: {str(e)}")
            logger.error(f"Traceback:\n{traceback.format_exc()}")
            return self.create_error_response(
                f"Error: {str(e)}",
                suggestion="patch",
                hint="Please try again or report the issue if it persists",
            )

    async def insert_text_file_contents(
        self,
        file_path: str,
        file_hash: str,
        contents: str,
        after: Optional[int] = None,
        before: Optional[int] = None,
        encoding: str = "utf-8",
    ) -> Dict[str, Any]:
        """Insert text content before or after a specific line in a file.

        Args:
            file_path (str): Path to the file to edit
            file_hash (str): Expected hash of the file before editing
            contents (str): Content to insert
            after (Optional[int]): Line number after which to insert content
            before (Optional[int]): Line number before which to insert content
            encoding (str, optional): File encoding. Defaults to "utf-8"

        Returns:
            Dict[str, Any]: Results containing:
                - result: "ok" or "error"
                - hash: New file hash if successful
                - reason: Error message if result is "error"
        """
        if (after is None and before is None) or (
            after is not None and before is not None
        ):
            return {
                "result": "error",
                "reason": "Exactly one of 'after' or 'before' must be specified",
                "hash": None,
            }

        try:
            (
                current_content,
                _,
                _,
                current_hash,
                total_lines,
                _,
            ) = await self.read_file_contents(
                file_path,
                encoding=encoding,
            )

            if current_hash != file_hash:
                return {
                    "result": "error",
                    "reason": "File hash mismatch - Please use get_text_file_contents tool to get current content and hash",
                    "hash": None,
                }

            # Split into lines, preserving line endings
            lines = current_content.splitlines(keepends=True)

            # Determine insertion point
            if after is not None:
                if after > total_lines:
                    return {
                        "result": "error",
                        "reason": f"Line number {after} is beyond end of file (total lines: {total_lines})",
                        "hash": None,
                    }
                insert_pos = after
            else:  # before must be set due to earlier validation
                assert before is not None
                if before > total_lines + 1:
                    return {
                        "result": "error",
                        "reason": f"Line number {before} is beyond end of file (total lines: {total_lines})",
                        "hash": None,
                    }
                insert_pos = before - 1

            # Ensure content ends with newline
            if not contents.endswith("\n"):
                contents += "\n"

            # Insert the content
            lines.insert(insert_pos, contents)

            # Join lines and write back to file
            final_content = "".join(lines)
            with open(file_path, "w", encoding=encoding) as f:
                f.write(final_content)

            # Calculate new hash
            new_hash = self.calculate_hash(final_content)

            return {
                "result": "ok",
                "hash": new_hash,
                "reason": None,
            }

        except FileNotFoundError:
            return {
                "result": "error",
                "reason": f"File not found: {file_path}",
                "hash": None,
            }
        except Exception as e:
            return {
                "result": "error",
                "reason": str(e),
                "hash": None,
            }

    async def delete_text_file_contents(
        self,
        request: DeleteTextFileContentsRequest,
    ) -> Dict[str, Any]:
        """Delete specified ranges from a text file with conflict detection.

        Args:
            request (DeleteTextFileContentsRequest): The request containing:
                - file_path: Path to the text file
                - file_hash: Expected hash of the file before editing
                - ranges: List of ranges to delete
                - encoding: Optional text encoding (default: utf-8)

        Returns:
            Dict[str, Any]: Results containing:
                - result: "ok" or "error"
                - hash: New file hash if successful
                - reason: Error message if result is "error"
        """
        self._validate_file_path(request.file_path)

        try:
            (
                current_content,
                _,
                _,
                current_hash,
                total_lines,
                _,
            ) = await self.read_file_contents(
                request.file_path,
                encoding=request.encoding or "utf-8",
            )

            # Check for conflicts
            if current_hash != request.file_hash:
                return {
                    request.file_path: {
                        "result": "error",
                        "reason": "File hash mismatch - Please use get_text_file_contents tool to get current content and hash",
                        "hash": current_hash,
                    }
                }

            # Split content into lines
            lines = current_content.splitlines(keepends=True)

            # Sort ranges in reverse order to handle line number shifts
            sorted_ranges = sorted(
                request.ranges,
                key=lambda x: (x.start, x.end or float("inf")),
                reverse=True,
            )

            # Validate ranges
            for i, range_ in enumerate(sorted_ranges):
                if range_.start < 1:
                    return {
                        request.file_path: {
                            "result": "error",
                            "reason": f"Invalid start line {range_.start}",
                            "hash": current_hash,
                        }
                    }

                if range_.end and range_.end < range_.start:
                    return {
                        request.file_path: {
                            "result": "error",
                            "reason": f"End line {range_.end} is less than start line {range_.start}",
                            "hash": current_hash,
                        }
                    }

                if range_.start > total_lines:
                    return {
                        request.file_path: {
                            "result": "error",
                            "reason": f"Start line {range_.start} exceeds file length {total_lines}",
                            "hash": current_hash,
                        }
                    }

                end = range_.end or total_lines
                if end > total_lines:
                    return {
                        request.file_path: {
                            "result": "error",
                            "reason": f"End line {end} exceeds file length {total_lines}",
                            "hash": current_hash,
                        }
                    }

                # Check for overlaps with next range
                if i + 1 < len(sorted_ranges):
                    next_range = sorted_ranges[i + 1]
                    next_end = next_range.end or total_lines
                    if next_end >= range_.start:
                        return {
                            request.file_path: {
                                "result": "error",
                                "reason": "Overlapping ranges detected",
                                "hash": current_hash,
                            }
                        }

            # Apply deletions
            for range_ in sorted_ranges:
                start_idx = range_.start - 1
                end_idx = range_.end if range_.end else len(lines)

                # Verify range content hash
                range_content = "".join(lines[start_idx:end_idx])
                if self.calculate_hash(range_content) != range_.range_hash:
                    return {
                        request.file_path: {
                            "result": "error",
                            "reason": f"Content hash mismatch for range {range_.start}-{range_.end}",
                            "hash": current_hash,
                        }
                    }

                del lines[start_idx:end_idx]

            # Write the final content back to file
            final_content = "".join(lines)
            with open(request.file_path, "w", encoding=request.encoding) as f:
                f.write(final_content)

            # Calculate new hash
            new_hash = self.calculate_hash(final_content)

            return {
                request.file_path: {
                    "result": "ok",
                    "hash": new_hash,
                    "reason": None,
                }
            }

        except FileNotFoundError:
            return {
                request.file_path: {
                    "result": "error",
                    "reason": f"File not found: {request.file_path}",
                    "hash": None,
                }
            }
        except Exception as e:
            return {
                request.file_path: {
                    "result": "error",
                    "reason": str(e),
                    "hash": None,
                }
            }



"""Handler for getting text file contents."""

import json
import os
from typing import Any, Dict, Sequence

from mcp.types import TextContent, Tool

from .base import BaseHandler


class GetTextFileContentsHandler(BaseHandler):
    """Handler for getting text file contents."""

    name = "get_text_file_contents"
    description = (
        "Read text file contents from multiple files and line ranges. "
        "Returns file contents with hashes for concurrency control and line numbers for reference. "
        "The hashes are used to detect conflicts when editing the files. File paths must be absolute."
    )

    def get_tool_description(self) -> Tool:
        """Get the tool description."""
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "files": {
                        "type": "array",
                        "description": "List of files and their line ranges to read",
                        "items": {
                            "type": "object",
                            "properties": {
                                "file_path": {
                                    "type": "string",
                                    "description": "Path to the text file. File path must be absolute.",
                                },
                                "ranges": {
                                    "type": "array",
                                    "description": "List of line ranges to read from the file",
                                    "items": {
                                        "type": "object",
                                        "properties": {
                                            "start": {
                                                "type": "integer",
                                                "description": "Starting line number (1-based)",
                                            },
                                            "end": {
                                                "type": ["integer", "null"],
                                                "description": "Ending line number (null for end of file)",
                                            },
                                        },
                                        "required": ["start"],
                                    },
                                },
                            },
                            "required": ["file_path", "ranges"],
                        },
                    },
                    "encoding": {
                        "type": "string",
                        "description": "Text encoding (default: 'utf-8')",
                        "default": "utf-8",
                    },
                },
                "required": ["files"],
            },
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        """Execute the tool with given arguments."""
        try:
            if "files" not in arguments:
                raise RuntimeError("Missing required argument: 'files'")

            for file_info in arguments["files"]:
                if not os.path.isabs(file_info["file_path"]):
                    raise RuntimeError(
                        f"File path must be absolute: {file_info['file_path']}"
                    )

            encoding = arguments.get("encoding", "utf-8")
            result = await self.editor.read_multiple_ranges(
                arguments["files"], encoding=encoding
            )
            response = result

            return [TextContent(type="text", text=json.dumps(response, indent=2))]

        except KeyError as e:
            raise RuntimeError(f"Missing required argument: '{e}'") from e
        except Exception as e:
            raise RuntimeError(f"Error processing request: {str(e)}") from e



"""Tests for delete text file functionality."""

from pathlib import Path

import pytest

from mcp_text_editor.models import DeleteTextFileContentsRequest, FileRange
from mcp_text_editor.text_editor import TextEditor


@pytest.fixture
def editor() -> TextEditor:
    """Create TextEditor instance."""
    return TextEditor()


@pytest.fixture
def test_file(tmp_path) -> Path:
    """Create a test file with sample content."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n")
    return test_file


@pytest.mark.asyncio
async def test_delete_single_line(editor, test_file):
    """Test deleting a single line from file."""
    # Get initial content and file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))
    original_lines = content.splitlines(keepends=True)

    # Get hash for line 2
    line2_hash = editor.calculate_hash(original_lines[1])

    # Create delete request
    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=2,
                end=2,
                range_hash=line2_hash,
            )
        ],
    )

    # Delete line 2
    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "ok"
    assert test_file.read_text() == "Line 1\nLine 3\nLine 4\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_multiple_lines(editor, test_file):
    """Test deleting multiple consecutive lines from file."""
    # Get initial content and file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))
    original_lines = content.splitlines(keepends=True)

    # Get hash for lines 2-4
    lines_hash = editor.calculate_hash("".join(original_lines[1:4]))

    # Create delete request
    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=2,
                end=4,
                range_hash=lines_hash,
            )
        ],
    )

    # Delete lines 2-4
    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "ok"
    assert test_file.read_text() == "Line 1\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_with_invalid_file_hash(editor, test_file):
    """Test deleting with an invalid file hash."""
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))
    original_lines = content.splitlines(keepends=True)
    line2_hash = editor.calculate_hash(original_lines[1])

    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash="invalid_hash",
        ranges=[
            FileRange(
                start=2,
                end=2,
                range_hash=line2_hash,
            )
        ],
    )

    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "error"
    assert "hash mismatch" in result[str(test_file)]["reason"].lower()
    assert test_file.read_text() == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_with_invalid_range_hash(editor, test_file):
    """Test deleting with an invalid range hash."""
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=2,
                end=2,
                range_hash="invalid_hash",
            )
        ],
    )

    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "error"
    assert "hash mismatch" in result[str(test_file)]["reason"].lower()
    assert test_file.read_text() == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_with_invalid_range(editor, test_file):
    """Test deleting with invalid line range."""
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))
    line2_hash = editor.calculate_hash("Line 2\n")

    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=2,
                end=1,  # Invalid: end before start
                range_hash=line2_hash,
            )
        ],
    )

    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "error"
    assert (
        "end line" in result[str(test_file)]["reason"].lower()
        and "less than start line" in result[str(test_file)]["reason"].lower()
    )
    assert test_file.read_text() == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_with_out_of_range(editor, test_file):
    """Test deleting lines beyond file length."""
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=10,
                end=12,
                range_hash="any_hash",  # Hash doesn't matter as it will fail before hash check
            )
        ],
    )

    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "error"
    assert (
        "start line" in result[str(test_file)]["reason"].lower()
        and "exceeds file length" in result[str(test_file)]["reason"].lower()
    )
    assert test_file.read_text() == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_with_overlapping_ranges(editor, test_file):
    """Test deleting with overlapping ranges."""
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Prepare hashes for the ranges
    lines = content.splitlines(keepends=True)
    range1_hash = editor.calculate_hash("".join(lines[1:3]))  # Lines 2-3
    range2_hash = editor.calculate_hash("".join(lines[2:4]))  # Lines 3-4

    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=2,
                end=3,
                range_hash=range1_hash,
            ),
            FileRange(
                start=3,
                end=4,
                range_hash=range2_hash,
            ),
        ],
    )

    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "error"
    assert "overlapping ranges" in result[str(test_file)]["reason"].lower()
    assert test_file.read_text() == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"


@pytest.mark.asyncio
async def test_delete_multiple_ranges(editor, test_file):
    """Test deleting multiple non-consecutive ranges."""
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Prepare hashes for the ranges
    lines = content.splitlines(keepends=True)
    range1_hash = editor.calculate_hash(lines[1])  # Line 2
    range2_hash = editor.calculate_hash(lines[3])  # Line 4

    request = DeleteTextFileContentsRequest(
        file_path=str(test_file),
        file_hash=file_hash,
        ranges=[
            FileRange(
                start=2,
                end=2,
                range_hash=range1_hash,
            ),
            FileRange(
                start=4,
                end=4,
                range_hash=range2_hash,
            ),
        ],
    )

    result = await editor.delete_text_file_contents(request)

    assert result[str(test_file)]["result"] == "ok"
    assert test_file.read_text() == "Line 1\nLine 3\nLine 5\n"



"""Test configuration and fixtures."""

import os
import tempfile
from typing import AsyncGenerator, Generator

import pytest
import pytest_asyncio
from mcp.server import Server

from mcp_text_editor.server import app


@pytest.fixture
def test_file() -> Generator[str, None, None]:
    """Create a temporary test file."""
    content = "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"
    with tempfile.NamedTemporaryFile(mode="w", delete=False) as f:
        f.write(content)
        file_path = f.name

    yield file_path

    # Cleanup
    if os.path.exists(file_path):
        os.unlink(file_path)


@pytest.fixture
def test_file_sjis() -> Generator[str, None, None]:
    """Create a temporary test file with Shift-JIS encoding."""
    # test1, test2, test3 in Japanese encoded in Shift-JIS
    content = b"\x83\x65\x83\x58\x83\x67\x31\x0a\x83\x65\x83\x58\x83\x67\x32\x0a\x83\x65\x83\x58\x83\x67\x33\x0a"
    with tempfile.NamedTemporaryFile(mode="wb", delete=False) as f:
        f.write(content)
        file_path = f.name

    yield file_path

    # Cleanup
    if os.path.exists(file_path):
        os.unlink(file_path)


class MockStream:
    """Mock stream for testing."""

    def __init__(self):
        self.data = []

    async def write(self, data: str) -> None:
        """Mock write method."""
        self.data.append(data)

    async def drain(self) -> None:
        """Mock drain method."""
        pass


@pytest_asyncio.fixture
async def mock_server() -> AsyncGenerator[tuple[Server, MockStream], None]:
    """Create a mock server for testing."""
    mock_write_stream = MockStream()
    yield app, mock_write_stream



"""Tests for the MCP Text Editor Server."""

import json
from pathlib import Path
from typing import List

import pytest
from mcp.server import stdio
from mcp.types import TextContent, Tool
from pytest_mock import MockerFixture

from mcp_text_editor.server import (
    GetTextFileContentsHandler,
    app,
    append_file_handler,
    call_tool,
    create_file_handler,
    delete_contents_handler,
    get_contents_handler,
    insert_file_handler,
    list_tools,
    main,
    patch_file_handler,
)


@pytest.mark.asyncio
async def test_list_tools():
    """Test tool listing."""
    tools: List[Tool] = await list_tools()
    assert len(tools) == 6

    # Verify GetTextFileContents tool
    get_contents_tool = next(
        (tool for tool in tools if tool.name == "get_text_file_contents"), None
    )
    assert get_contents_tool is not None
    assert "file" in get_contents_tool.description.lower()
    assert "contents" in get_contents_tool.description.lower()


@pytest.mark.asyncio
async def test_get_contents_empty_files():
    """Test get_contents handler with empty files list."""
    arguments = {"files": []}
    result = await get_contents_handler.run_tool(arguments)
    assert len(result) == 1
    assert result[0].type == "text"
    # Should return empty JSON object
    assert json.loads(result[0].text) == {}


@pytest.mark.asyncio
async def test_unknown_tool_handler():
    """Test handling of unknown tool name."""
    with pytest.raises(ValueError) as excinfo:
        await call_tool("unknown_tool", {})
    assert "Unknown tool: unknown_tool" in str(excinfo.value)


@pytest.mark.asyncio
async def test_get_contents_handler(test_file):
    """Test GetTextFileContents handler."""
    args = {"files": [{"file_path": test_file, "ranges": [{"start": 1, "end": 3}]}]}
    result = await get_contents_handler.run_tool(args)
    assert len(result) == 1
    assert isinstance(result[0], TextContent)
    content = json.loads(result[0].text)
    assert test_file in content
    range_result = content[test_file]["ranges"][0]
    assert "content" in range_result
    assert "start" in range_result
    assert "end" in range_result
    assert "file_hash" in content[test_file]
    assert "total_lines" in range_result
    assert "content_size" in range_result


@pytest.mark.asyncio
async def test_get_contents_handler_invalid_file(test_file):
    """Test GetTextFileContents handler with invalid file."""
    # Convert relative path to absolute
    nonexistent_path = str(Path("nonexistent.txt").absolute())
    args = {"files": [{"file_path": nonexistent_path, "ranges": [{"start": 1}]}]}
    with pytest.raises(RuntimeError) as exc_info:
        await get_contents_handler.run_tool(args)
    assert "File not found" in str(exc_info.value)


@pytest.mark.asyncio
async def test_call_tool_get_contents(test_file):
    """Test call_tool with GetTextFileContents."""
    args = {"files": [{"file_path": test_file, "ranges": [{"start": 1, "end": 3}]}]}
    result = await call_tool("get_text_file_contents", args)
    assert len(result) == 1
    assert isinstance(result[0], TextContent)
    content = json.loads(result[0].text)
    assert test_file in content
    range_result = content[test_file]["ranges"][0]
    assert "content" in range_result
    assert "start" in range_result
    assert "end" in range_result
    assert "file_hash" in content[test_file]
    assert "total_lines" in range_result
    assert "content_size" in range_result


@pytest.mark.asyncio
async def test_call_tool_unknown():
    """Test call_tool with unknown tool."""
    with pytest.raises(ValueError) as exc_info:
        await call_tool("UnknownTool", {})
    assert "Unknown tool" in str(exc_info.value)


@pytest.mark.asyncio
async def test_call_tool_error_handling():
    """Test call_tool error handling."""
    # Test with invalid arguments
    with pytest.raises(RuntimeError) as exc_info:
        await call_tool("get_text_file_contents", {"invalid": "args"})
    assert "Missing required argument" in str(exc_info.value)

    # Convert relative path to absolute
    nonexistent_path = str(Path("nonexistent.txt").absolute())
    with pytest.raises(RuntimeError) as exc_info:
        await call_tool(
            "get_text_file_contents",
            {"files": [{"file_path": nonexistent_path, "ranges": [{"start": 1}]}]},
        )
    assert "File not found" in str(exc_info.value)


@pytest.mark.asyncio
async def test_get_contents_handler_legacy_missing_args():
    """Test GetTextFileContents handler with legacy single file request missing arguments."""
    with pytest.raises(RuntimeError) as exc_info:
        await get_contents_handler.run_tool({})
    assert "Missing required argument: 'files'" in str(exc_info.value)


@pytest.mark.asyncio
async def test_main_stdio_server_error(mocker: MockerFixture):
    """Test main function with stdio_server error."""
    # Mock the stdio_server to raise an exception
    mock_stdio = mocker.patch.object(stdio, "stdio_server")
    mock_stdio.side_effect = Exception("Stdio server error")

    with pytest.raises(Exception) as exc_info:
        await main()
    assert "Stdio server error" in str(exc_info.value)


@pytest.mark.asyncio
async def test_main_run_error(mocker: MockerFixture):
    """Test main function with app.run error."""
    # Mock the stdio_server context manager
    mock_stdio = mocker.patch.object(stdio, "stdio_server")
    mock_context = mocker.MagicMock()
    mock_context.__aenter__.return_value = (mocker.MagicMock(), mocker.MagicMock())
    mock_stdio.return_value = mock_context

    # Mock app.run to raise an exception
    mock_run = mocker.patch.object(app, "run")
    mock_run.side_effect = Exception("App run error")

    with pytest.raises(Exception) as exc_info:
        await main()
    assert "App run error" in str(exc_info.value)


@pytest.mark.asyncio
async def test_get_contents_relative_path():
    """Test GetTextFileContents with relative path."""
    handler = GetTextFileContentsHandler()
    with pytest.raises(RuntimeError, match="File path must be absolute:.*"):
        await handler.run_tool(
            {
                "files": [
                    {"file_path": "relative/path/file.txt", "ranges": [{"start": 1}]}
                ]
            }
        )


@pytest.mark.asyncio
async def test_get_contents_absolute_path():
    """Test GetTextFileContents with absolute path."""
    handler = GetTextFileContentsHandler()
    abs_path = str(Path("/absolute/path/file.txt").absolute())

    # Define mock as async function
    async def mock_read_multiple_ranges(*args, **kwargs):
        return []

    # Set up mock
    handler.editor.read_multiple_ranges = mock_read_multiple_ranges

    result = await handler.run_tool(
        {"files": [{"file_path": abs_path, "ranges": [{"start": 1}]}]}
    )
    assert isinstance(result[0], TextContent)


@pytest.mark.asyncio
async def test_call_tool_general_exception():
    """Test call_tool with a general exception."""
    # Patch get_contents_handler.run_tool to raise a general exception
    original_run_tool = get_contents_handler.run_tool

    async def mock_run_tool(args):
        raise Exception("Unexpected error")

    try:
        get_contents_handler.run_tool = mock_run_tool
        with pytest.raises(RuntimeError) as exc_info:
            await call_tool("get_text_file_contents", {"files": []})
        assert "Error executing command: Unexpected error" in str(exc_info.value)
    finally:
        # Restore original method
        get_contents_handler.run_tool = original_run_tool


@pytest.mark.asyncio
async def test_call_tool_all_handlers(mocker: MockerFixture):
    """Test call_tool with all handlers."""
    # Mock run_tool for each handler
    handlers = [
        create_file_handler,
        append_file_handler,
        delete_contents_handler,
        insert_file_handler,
        patch_file_handler,
    ]

    # Setup mocks for all handlers
    async def mock_run_tool(args):
        return [TextContent(text="mocked response", type="text")]

    for handler in handlers:
        mock = mocker.patch.object(handler, "run_tool")
        mock.side_effect = mock_run_tool

    # Test each handler
    for handler in handlers:
        result = await call_tool(handler.name, {"test": "args"})
        assert len(result) == 1
        assert isinstance(result[0], TextContent)
        assert result[0].text == "mocked response"



"""Tests for error hints and suggestions functionality."""

import pytest

from mcp_text_editor.text_editor import TextEditor


@pytest.fixture
def editor():
    """Create TextEditor instance."""
    return TextEditor()


@pytest.mark.asyncio
async def test_file_not_found_hint(editor, tmp_path):
    """Test hints when file is not found."""
    non_existent = tmp_path / "non_existent.txt"

    result = await editor.edit_file_contents(
        str(non_existent),
        "non_empty_hash",
        [{"start": 1, "contents": "test", "range_hash": ""}],
    )

    assert result["result"] == "error"
    assert "File not found" in result["reason"]
    assert result["suggestion"] == "append"
    assert "append_text_file_contents" in result["hint"]


@pytest.mark.asyncio
async def test_hash_mismatch_hint(editor, tmp_path):
    """Test hints when file hash doesn't match."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("original content\\n")

    result = await editor.edit_file_contents(
        str(test_file),
        "wrong_hash",
        [{"start": 1, "contents": "new content", "range_hash": ""}],
    )

    assert result["result"] == "error"
    assert "hash mismatch" in result["reason"].lower()
    assert result["suggestion"] == "patch"
    assert "get_text_file_contents tool" in result["hint"]


@pytest.mark.asyncio
async def test_overlapping_patches_hint(editor, tmp_path):
    """Test hints when patches overlap."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("line1\\nline2\\nline3\\n")

    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 1,
                "end": 2,
                "contents": "new1\\n",
                "range_hash": editor.calculate_hash("line1\\nline2\\n"),
            },
            {
                "start": 2,
                "end": 3,
                "contents": "new2\\n",
                "range_hash": editor.calculate_hash("line2\\nline3\\n"),
            },
        ],
    )

    assert result["result"] == "error"
    assert "overlap" in result["reason"].lower()
    assert result["suggestion"] == "patch"
    assert "not overlap" in result["hint"].lower()


@pytest.mark.asyncio
async def test_io_error_hint(editor, tmp_path, monkeypatch):
    """Test hints when IO error occurs."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("original content\\n")

    def mock_open(*args, **kwargs):
        raise IOError("Test IO Error")

    monkeypatch.setattr("builtins.open", mock_open)

    result = await editor.edit_file_contents(
        str(test_file), "", [{"start": 1, "contents": "new content\\n"}]
    )

    assert result["result"] == "error"
    assert "Error editing file" in result["reason"]
    assert result["suggestion"] == "patch"
    assert "permissions" in result["hint"].lower()


@pytest.mark.asyncio
async def test_empty_content_delete_hint(editor, tmp_path):
    """Test hints when trying to delete content using empty patch."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("original\\n")

    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 1,
                "end": 1,
                "contents": "",
                "range_hash": editor.calculate_hash("original\\n"),
            }
        ],
    )

    assert result["result"] == "ok"  # Note: It's "ok" but suggests using delete
    assert result["suggestion"] == "delete"
    assert "delete_text_file_contents" in result["hint"]


@pytest.mark.asyncio
async def test_append_suggestion_for_new_file(editor, tmp_path):
    """Test suggestion to use append for new files."""
    test_file = tmp_path / "new.txt"

    result = await editor.edit_file_contents(
        str(test_file),
        "",
        [{"start": 1, "contents": "new content\\n", "range_hash": ""}],
    )

    assert result["result"] == "ok"
    assert result["suggestion"] == "append"
    assert "append_text_file_contents" in result["hint"]



"""Test cases for create_text_file handler."""

import os
from typing import Any, Dict, Generator

import pytest

from mcp_text_editor.server import CreateTextFileHandler

# Initialize handlers for tests
create_file_handler = CreateTextFileHandler()


@pytest.fixture
def test_dir(tmp_path: str) -> str:
    """Create a temporary directory for test files."""
    return str(tmp_path)


@pytest.fixture
def cleanup_files() -> Generator[None, None, None]:
    """Clean up any test files after each test."""
    yield
    # Add cleanup code if needed


@pytest.mark.asyncio
async def test_create_text_file_success(test_dir: str, cleanup_files: None) -> None:
    """Test successful creation of a new text file."""
    test_file = os.path.join(test_dir, "new_file.txt")
    content = "Hello, World!\n"

    # Create file using handler
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": content,
    }
    response = await create_file_handler.run_tool(arguments)

    # Check if file was created with correct content
    assert os.path.exists(test_file)
    with open(test_file, "r", encoding="utf-8") as f:
        assert f.read() == content

    # Parse response to check success
    assert len(response) == 1
    result = response[0].text
    assert '"result": "ok"' in result


@pytest.mark.asyncio
async def test_create_text_file_exists(test_dir: str, cleanup_files: None) -> None:
    """Test attempting to create a file that already exists."""
    test_file = os.path.join(test_dir, "existing_file.txt")

    # Create file first
    with open(test_file, "w", encoding="utf-8") as f:
        f.write("Existing content\n")

    # Try to create file using handler
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": "New content\n",
    }

    # Should raise error because file exists
    with pytest.raises(RuntimeError):
        await create_file_handler.run_tool(arguments)


@pytest.mark.asyncio
async def test_create_text_file_relative_path(
    test_dir: str, cleanup_files: None
) -> None:
    """Test attempting to create a file with a relative path."""
    # Try to create file using relative path
    arguments: Dict[str, Any] = {
        "file_path": "relative_path.txt",
        "contents": "Some content\n",
    }

    # Should raise error because path is not absolute
    with pytest.raises(RuntimeError) as exc_info:
        await create_file_handler.run_tool(arguments)
    assert "File path must be absolute" in str(exc_info.value)


@pytest.mark.asyncio
async def test_create_text_file_missing_args() -> None:
    """Test creating a file with missing arguments."""
    # Test missing path
    with pytest.raises(RuntimeError) as exc_info:
        await create_file_handler.run_tool({"contents": "content\n"})
    assert "Missing required argument: file_path" in str(exc_info.value)

    # Test missing contents
    with pytest.raises(RuntimeError) as exc_info:
        await create_file_handler.run_tool({"file_path": "/absolute/path.txt"})
    assert "Missing required argument: contents" in str(exc_info.value)


@pytest.mark.asyncio
async def test_create_text_file_custom_encoding(
    test_dir: str, cleanup_files: None
) -> None:
    """Test creating a file with custom encoding."""
    test_file = os.path.join(test_dir, "encoded_file.txt")
    content = "こんにちは\n"  # Japanese text

    # Create file using handler with specified encoding
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": content,
        "encoding": "utf-8",
    }
    response = await create_file_handler.run_tool(arguments)

    # Check if file was created with correct content
    assert os.path.exists(test_file)
    with open(test_file, "r", encoding="utf-8") as f:
        assert f.read() == content

    # Parse response to check success
    assert len(response) == 1
    result = response[0].text
    assert '"result": "ok"' in result



"""Tests for error response creation and hint/suggestion functionality."""

import pytest

from mcp_text_editor.text_editor import TextEditor


@pytest.fixture
def editor():
    """Create TextEditor instance."""
    return TextEditor()


def test_create_error_response_basic(editor):
    """Test basic error response without hint/suggestion."""
    response = editor.create_error_response("Test error")
    assert response["result"] == "error"
    assert response["reason"] == "Test error"
    assert response["file_hash"] is None
    assert "hint" not in response
    assert "suggestion" not in response


def test_create_error_response_with_hint_suggestion(editor):
    """Test error response with hint and suggestion."""
    response = editor.create_error_response(
        "Test error", suggestion="append", hint="Please use append_text_file_contents"
    )
    assert response["result"] == "error"
    assert response["reason"] == "Test error"
    assert response["suggestion"] == "append"
    assert response["hint"] == "Please use append_text_file_contents"


def test_create_error_response_with_file_path(editor):
    """Test error response with file path."""
    response = editor.create_error_response(
        "Test error",
        file_path="/test/file.txt",
        suggestion="patch",
        hint="Please try again",
    )
    assert "/test/file.txt" in response
    assert response["/test/file.txt"]["result"] == "error"
    assert response["/test/file.txt"]["reason"] == "Test error"
    assert response["/test/file.txt"]["suggestion"] == "patch"
    assert response["/test/file.txt"]["hint"] == "Please try again"


def test_create_error_response_with_hash(editor):
    """Test error response with content hash."""
    test_hash = "test_hash_value"
    response = editor.create_error_response("Test error", content_hash=test_hash)
    assert response["result"] == "error"
    assert response["reason"] == "Test error"
    assert response["file_hash"] == test_hash



"""Data models for the MCP Text Editor Server."""

from typing import Dict, List, Optional

from pydantic import BaseModel, Field, field_validator, model_validator


class GetTextFileContentsRequest(BaseModel):
    """Request model for getting text file contents."""

    file_path: str = Field(..., description="Path to the text file")
    start: int = Field(1, description="Starting line number (1-based)")
    end: Optional[int] = Field(None, description="Ending line number (inclusive)")


class GetTextFileContentsResponse(BaseModel):
    """Response model for getting text file contents."""

    contents: str = Field(..., description="File contents")
    start: int = Field(..., description="Starting line number")
    end: int = Field(..., description="Ending line number")
    hash: str = Field(..., description="Hash of the contents")


class EditPatch(BaseModel):
    """Model for a single edit patch operation."""

    start: int = Field(1, description="Starting line for edit")
    end: Optional[int] = Field(None, description="Ending line for edit")
    contents: str = Field(..., description="New content to insert")
    range_hash: Optional[str] = Field(
        None,  # None for new patches, must be explicitly set
        description="Hash of content being replaced. Empty string for insertions.",
    )

    @model_validator(mode="after")
    def validate_range_hash(self) -> "EditPatch":
        """Validate that range_hash is set and handle end field validation."""
        # range_hash must be explicitly set
        if self.range_hash is None:
            raise ValueError("range_hash is required")

        # For safety, convert None to the special range hash value
        if self.end is None and self.range_hash != "":
            # Special case: patch with end=None is allowed
            pass

        return self


class EditFileOperation(BaseModel):
    """Model for individual file edit operation."""

    path: str = Field(..., description="Path to the file")
    hash: str = Field(..., description="Hash of original contents")
    patches: List[EditPatch] = Field(..., description="Edit operations to apply")


class EditResult(BaseModel):
    """Model for edit operation result."""

    result: str = Field(..., description="Operation result (ok/error)")
    reason: Optional[str] = Field(None, description="Error message if applicable")
    hash: Optional[str] = Field(
        None, description="Current content hash (None for missing files)"
    )

    @model_validator(mode="after")
    def validate_error_result(self) -> "EditResult":
        """Remove hash when result is error."""
        if self.result == "error":
            object.__setattr__(self, "hash", None)
        return self

    def to_dict(self) -> Dict:
        """Convert EditResult to a dictionary."""
        result = {"result": self.result}
        if self.reason is not None:
            result["reason"] = self.reason
        if self.hash is not None:
            result["hash"] = self.hash
        return result


class EditTextFileContentsRequest(BaseModel):
    """Request model for editing text file contents.

    Example:
    {
        "files": [
            {
                "path": "/path/to/file",
                "hash": "abc123...",
                "patches": [
                    {
                        "start": 1,  # default: 1 (top of file)
                        "end": null,  # default: null (end of file)
                        "contents": "new content"
                    }
                ]
            }
        ]
    }
    """

    files: List[EditFileOperation] = Field(..., description="List of file operations")


class FileRange(BaseModel):
    """Represents a line range in a file."""

    start: int = Field(..., description="Starting line number (1-based)")
    end: Optional[int] = Field(
        None, description="Ending line number (null for end of file)"
    )
    range_hash: Optional[str] = Field(
        None, description="Hash of the content to be deleted"
    )


class FileRanges(BaseModel):
    """Represents a file and its line ranges."""

    file_path: str = Field(..., description="Path to the text file")
    ranges: List[FileRange] = Field(
        ..., description="List of line ranges to read from the file"
    )


class InsertTextFileContentsRequest(BaseModel):
    """Request model for inserting text into a file.

    Example:
    {
        "path": "/path/to/file",
        "file_hash": "abc123...",
        "after": 5,  # Insert after line 5
        "contents": "new content"
    }
    or
    {
        "path": "/path/to/file",
        "file_hash": "abc123...",
        "before": 5,  # Insert before line 5
        "contents": "new content"
    }
    """

    path: str = Field(..., description="Path to the text file")
    file_hash: str = Field(..., description="Hash of original contents")
    after: Optional[int] = Field(
        None, description="Line number after which to insert content"
    )
    before: Optional[int] = Field(
        None, description="Line number before which to insert content"
    )
    encoding: Optional[str] = Field(
        "utf-8", description="Text encoding (default: 'utf-8')"
    )
    contents: str = Field(..., description="Content to insert")

    @model_validator(mode="after")
    def validate_position(self) -> "InsertTextFileContentsRequest":
        """Validate that exactly one of after or before is specified."""
        if (self.after is None and self.before is None) or (
            self.after is not None and self.before is not None
        ):
            raise ValueError("Exactly one of 'after' or 'before' must be specified")
        return self

    @field_validator("after", "before")
    def validate_line_number(cls, v) -> Optional[int]:
        """Validate that line numbers are positive."""
        if v is not None and v < 1:
            raise ValueError("Line numbers must be positive")
        return v


class DeleteTextFileContentsRequest(BaseModel):
    """Request model for deleting text from a file.
    Example:
    {
        "file_path": "/path/to/file",
        "file_hash": "abc123...",
        "ranges": [
            {
                "start": 5,
                "end": 10,
                "range_hash": "def456..."
            }
        ]
    }
    """

    file_path: str = Field(..., description="Path to the text file")
    file_hash: str = Field(..., description="Hash of original contents")
    ranges: List[FileRange] = Field(..., description="List of ranges to delete")
    encoding: Optional[str] = Field(
        "utf-8", description="Text encoding (default: 'utf-8')"
    )


class PatchTextFileContentsRequest(BaseModel):
    """Request model for patching text in a file.
    Example:
    {
        "file_path": "/path/to/file",
        "file_hash": "abc123...",
        "patches": [
            {
                "start": 5,
                "end": 10,
                "contents": "new content",
                "range_hash": "def456..."
            }
        ]
    }
    """

    file_path: str = Field(..., description="Path to the text file")
    file_hash: str = Field(..., description="Hash of original contents")
    patches: List[EditPatch] = Field(..., description="List of patches to apply")
    encoding: Optional[str] = Field(
        "utf-8", description="Text encoding (default: 'utf-8')"
    )



"""Tests for the TextEditor class."""

import pytest

from mcp_text_editor.text_editor import EditPatch, TextEditor


@pytest.fixture
def editor():
    """Create TextEditor instance."""
    return TextEditor()


@pytest.mark.asyncio
async def test_edit_file_with_edit_patch_object(editor, tmp_path):
    """Test editing a file using EditPatch object."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("line1\nline2\nline3\n")
    file_hash = editor.calculate_hash(test_file.read_text())
    first_line_content = "line1\n"

    # Create an EditPatch object
    patch = EditPatch(
        start=1,
        end=1,
        contents="new line\n",
        range_hash=editor.calculate_hash(first_line_content),
    )

    result = await editor.edit_file_contents(str(test_file), file_hash, [patch])

    assert result["result"] == "ok"
    assert test_file.read_text() == "new line\nline2\nline3\n"


@pytest.fixture
def test_file(tmp_path):
    """Create a temporary test file."""
    file_path = tmp_path / "test.txt"
    content = "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"
    file_path.write_text(content)
    return file_path


@pytest.mark.asyncio
async def test_directory_creation_error(editor, tmp_path, mocker):
    """Test file creation when parent directory creation fails."""
    test_dir = tmp_path / "test_dir"
    test_file = test_dir / "test.txt"

    # Mock os.makedirs to raise an OSError
    mocker.patch("os.makedirs", side_effect=OSError("Permission denied"))

    result = await editor.edit_file_contents(
        str(test_file), "", [EditPatch(contents="test content\n", range_hash="")]
    )

    assert result["result"] == "error"
    assert "Failed to create directory" in result["reason"]
    assert result["file_hash"] is None


@pytest.mark.asyncio
async def test_missing_range_hash(editor, test_file):
    """Test editing without required range hash."""
    _, _, _, file_hash, _, _ = await editor.read_file_contents(test_file)

    # Try to edit without range_hash
    with pytest.raises(ValueError, match="range_hash is required"):
        EditPatch(start=2, end=2, contents="New content\n", range_hash=None)

    with pytest.raises(ValueError, match="range_hash is required"):
        # Trying with missing range_hash field should also raise
        EditPatch(start=2, end=2, contents="New content\n")


@pytest.fixture
def test_invalid_encoding_file(tmp_path):
    """Create a temporary file with a custom encoding to test encoding errors."""
    file_path = tmp_path / "invalid_encoding.txt"
    # Create Shift-JIS encoded file that will fail to decode with UTF-8
    test_data = bytes(
        [0x83, 0x65, 0x83, 0x58, 0x83, 0x67, 0x0A]
    )  # "テスト\n" in Shift-JIS
    with open(file_path, "wb") as f:
        f.write(test_data)
    return str(file_path)


@pytest.mark.asyncio
async def test_calculate_hash(editor):
    """Test hash calculation."""
    content = "test content"
    hash1 = editor.calculate_hash(content)
    hash2 = editor.calculate_hash(content)
    assert hash1 == hash2
    assert isinstance(hash1, str)
    assert len(hash1) == 64  # SHA-256 hash length


@pytest.mark.asyncio
async def test_read_file_contents(editor, test_file):
    """Test reading file contents."""
    # Test reading entire file
    content, start, end, hash_value, total_lines, size = (
        await editor.read_file_contents(str(test_file))
    )
    assert content == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"
    assert start == 1
    assert end == 5
    assert isinstance(hash_value, str)
    assert total_lines == 5
    assert size == len(content)

    # Test reading specific lines
    content, start, end, hash_value, total_lines, size = (
        await editor.read_file_contents(str(test_file), start=2, end=4)
    )
    assert content == "Line 2\nLine 3\nLine 4\n"
    assert start == 2
    assert end == 4
    assert isinstance(hash_value, str)
    assert total_lines == 5  # Total lines in file should remain the same
    assert size == len(content)  # Size should match the selected content


@pytest.mark.asyncio
async def test_encoding_error(editor, test_invalid_encoding_file):
    """Test handling of encoding errors when reading a file with incorrect encoding."""
    # Try to read Shift-JIS file with UTF-8 encoding
    with pytest.raises(UnicodeDecodeError) as excinfo:
        await editor.read_file_contents(test_invalid_encoding_file, encoding="utf-8")

    assert "Failed to decode file" in str(excinfo.value)
    assert "utf-8" in str(excinfo.value)

    # Try to read Shift-JIS file with incorrect encoding in edit_file_contents
    result = await editor.edit_file_contents(
        test_invalid_encoding_file,
        "dummy_hash",
        [{"start": 1, "contents": "test", "range_hash": "dummy_hash"}],
        encoding="utf-8",
    )

    assert result["result"] == "error"
    assert "Error editing file" in result["reason"]
    assert "decode" in result["reason"].lower()


@pytest.mark.asyncio
async def test_create_new_file(editor, tmp_path):
    """Test creating a new file."""
    new_file = tmp_path / "new_file.txt"
    content = "New file content\nLine 2\n"

    # Test creating a new file
    result = await editor.edit_file_contents(
        str(new_file),
        "",  # No hash for new file
        [
            {"start": 1, "contents": content, "range_hash": ""}
        ],  # Empty range_hash for new files
    )
    assert result["result"] == "ok"
    assert new_file.read_text() == content


@pytest.mark.asyncio
async def test_update_file(editor, tmp_path):
    """Test updating an existing file."""
    # Create a test file
    test_file = tmp_path / "test_update.txt"
    original_content = "Line 1\nLine 2\nLine 3\n"
    test_file.write_text(original_content)

    # Read the content and get hash
    content, start, end, file_hash, total_lines, size = await editor.read_file_contents(
        str(test_file)
    )

    # Update the second line
    new_content = "Updated Line 2\n"
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 2,
                "end": 2,
                "contents": new_content,
                "range_hash": editor.calculate_hash("Line 2\n"),
            }
        ],
    )

    assert result["result"] == "ok"
    assert test_file.read_text() == "Line 1\nUpdated Line 2\nLine 3\n"


@pytest.mark.asyncio
async def test_create_file_in_new_directory(editor, tmp_path):
    """Test creating a file in a new directory structure."""
    # Test file in a new directory structure
    new_file = tmp_path / "subdir" / "nested" / "test.txt"
    content = "Content in nested directory\n"

    result = await editor.edit_file_contents(
        str(new_file),
        "",  # No hash for new file
        [
            {"start": 1, "contents": content, "range_hash": ""}
        ],  # Empty range_hash for new file
    )

    assert result["result"] == "ok"
    assert new_file.read_text() == content


@pytest.mark.asyncio
async def test_file_hash_mismatch(editor, tmp_path):
    """Test handling of file hash mismatch."""
    # Create a test file
    test_file = tmp_path / "test_hash_mismatch.txt"
    original_content = "Line 1\nLine 2\nLine 3\n"
    test_file.write_text(original_content)

    result = await editor.edit_file_contents(
        str(test_file),
        "invalid_hash",  # Wrong hash
        [
            {
                "start": 2,
                "end": 2,
                "contents": "Updated Line\n",
                "range_hash": editor.calculate_hash("Line 2\n"),
            }
        ],
    )

    assert result["result"] == "error"
    assert "hash mismatch" in result["reason"].lower()
    assert test_file.read_text() == original_content  # File should remain unchanged
    assert test_file.read_text() == original_content  # File should remain unchanged


@pytest.mark.asyncio
async def test_path_traversal_prevention(editor, tmp_path):
    """Test prevention of path traversal attacks."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("Some content\n")
    unsafe_path = str(test_file) + "/.."  # Try to traverse up

    # Test read operation
    with pytest.raises(ValueError) as excinfo:
        await editor.read_file_contents(unsafe_path)
    assert "Path traversal not allowed" in str(excinfo.value)

    # Test write operation
    with pytest.raises(ValueError) as excinfo:
        await editor.edit_file_contents(
            unsafe_path,
            "",
            [{"start": 1, "contents": "malicious content\n", "range_hash": None}],
        )
    assert "Path traversal not allowed" in str(excinfo.value)


@pytest.mark.asyncio
async def test_overlapping_patches(editor, tmp_path):
    """Test handling of overlapping patches."""
    # Create a test file
    test_file = tmp_path / "test_overlap.txt"
    original_content = "Line 1\nLine 2\nLine 3\nLine 4\n"
    test_file.write_text(original_content)

    # Get file hash
    content, start, end, file_hash, total_lines, size = await editor.read_file_contents(
        str(test_file)
    )

    # Try to apply overlapping patches
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 1,
                "end": 2,
                "contents": "New Line 1-2\n",
                "range_hash": editor.calculate_hash("Line 1\nLine 2\n"),
            },
            {
                "start": 2,
                "end": 3,
                "contents": "New Line 2-3\n",
                "range_hash": editor.calculate_hash("Line 2\nLine 3\n"),
            },
        ],
    )

    assert result["result"] == "error"
    assert "overlap" in result["reason"].lower()
    assert test_file.read_text() == original_content  # File should remain unchanged


@pytest.mark.asyncio
async def test_empty_content_handling(editor, tmp_path):
    """Test handling of empty file content."""
    # Create an empty test file
    test_file = tmp_path / "empty.txt"
    test_file.write_text("")

    # Read empty file
    content, start, end, file_hash, total_lines, size = await editor.read_file_contents(
        str(test_file)
    )
    assert content == ""
    assert total_lines == 0
    assert size == 0

    # Write to empty file (treat it as a new file)
    result = await editor.edit_file_contents(
        str(test_file),
        "",  # No hash for empty file
        [
            {"line_start": 1, "contents": "New content\n", "range_hash": ""}
        ],  # Empty range_hash for new files
    )

    assert result["result"] == "ok"
    assert test_file.read_text() == "New content\n"


@pytest.mark.asyncio
async def test_read_multiple_ranges_line_exceed(editor, tmp_path):
    """Test reading multiple ranges with exceeding line numbers."""
    # Create a test file
    test_file = tmp_path / "test.txt"
    content = "Line 1\nLine 2\nLine 3\n"
    test_file.write_text(content)

    # Request ranges that exceed file length
    ranges = [
        {
            "file_path": str(test_file),
            "ranges": [{"start": 4, "end": None}, {"start": 1, "end": 2}],
        }
    ]

    result = await editor.read_multiple_ranges(ranges)

    # Check the exceeded range
    assert len(result[str(test_file)]["ranges"]) == 2
    # First range (exceeded)
    assert result[str(test_file)]["ranges"][0]["content"] == ""
    assert result[str(test_file)]["ranges"][0]["start"] == 4
    assert result[str(test_file)]["ranges"][0]["end"] == 4
    assert result[str(test_file)]["ranges"][0]["content_size"] == 0
    # Second range (normal)
    assert result[str(test_file)]["ranges"][1]["content"] == "Line 1\nLine 2\n"


@pytest.mark.asyncio
async def test_directory_creation_failure(editor, tmp_path):
    """Test failure in directory creation."""
    # Create a file in place of a directory to cause mkdir to fail
    base_dir = tmp_path / "blocked"
    base_dir.write_text("")
    test_file = base_dir / "subdir" / "test.txt"

    result = await editor.edit_file_contents(
        str(test_file),
        "",  # New file
        [{"line_start": 1, "contents": "test content\n", "range_hash": None}],
    )

    assert result["result"] == "error"
    assert "Failed to create directory" in result["reason"]
    assert result["file_hash"] is None


@pytest.mark.asyncio
async def test_invalid_encoding_file_operations(editor, tmp_path):
    """Test handling of files with invalid encoding during various operations."""
    test_file = tmp_path / "invalid_encoding.txt"
    # Create a file with Shift-JIS content that will fail UTF-8 decoding
    test_data = bytes([0x83, 0x65, 0x83, 0x58, 0x83, 0x67, 0x0A])  # シフトJISのデータ
    with open(test_file, "wb") as f:
        f.write(test_data)

    # Test encoding error in file operations
    result = await editor.edit_file_contents(
        str(test_file),
        "",  # hash doesn't matter as it will fail before hash check
        [{"line_start": 1, "contents": "new content\n", "range_hash": None}],
        encoding="utf-8",
    )

    assert result["result"] == "error"
    assert "Error editing file" in result["reason"]
    assert "decode" in result["reason"].lower()


@pytest.mark.asyncio
async def test_create_file_directory_error(editor, tmp_path, monkeypatch):
    """Test creating a file when directory creation fails."""
    # Create a path with multiple nested directories
    deep_path = tmp_path / "deeply" / "nested" / "path" / "test.txt"

    # Mock os.makedirs to raise an OSError
    def mock_makedirs(*args, **kwargs):
        raise OSError("Permission denied")

    monkeypatch.setattr("os.makedirs", mock_makedirs)

    # Attempt to create a new file
    result = await editor.edit_file_contents(
        str(deep_path),
        "",  # Empty hash for new file
        [
            {
                "start": 1,
                "contents": "test content\n",
            }
        ],
    )

    # Verify proper error handling
    assert result["result"] == "error"
    assert "Failed to create directory" in result["reason"]
    assert "Permission denied" in result["reason"]
    assert result["file_hash"] is None
    assert "content" not in result


@pytest.mark.asyncio
async def test_create_file_with_empty_directory(editor, tmp_path):
    """Test creating a file when parent directory is an empty string."""
    # Create a file in the current directory (no parent directory)
    file_path = tmp_path / "test.txt"

    # Attempt to create a new file
    result = await editor.edit_file_contents(
        str(file_path),
        "",  # Empty hash for new file
        [
            {
                "start": 1,
                "contents": "test content\n",
                "range_hash": "",  # Empty range_hash for new files
            }
        ],
    )

    # Verify successful file creation
    assert result["result"] == "ok"
    assert file_path.read_text() == "test content\n"
    assert result["file_hash"] is not None


@pytest.mark.asyncio
async def test_file_write_permission_error(editor, tmp_path):
    """Test file write permission error handling."""
    # Create a test file
    test_file = tmp_path / "readonly.txt"
    test_file.write_text("original content\n")
    test_file.chmod(0o444)  # Make file read-only

    # Get file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Attempt to modify read-only file
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 1,
                "end": 1,
                "contents": "new content\n",
                "range_hash": editor.calculate_hash("original content\n"),
            }
        ],
    )

    # Verify proper error handling
    assert result["result"] == "error"
    assert "Error editing file" in result["reason"]
    assert "Permission" in result["reason"]
    assert result["file_hash"] is None
    assert "content" not in result


@pytest.mark.asyncio
async def test_edit_file_with_none_line_end(editor, tmp_path):
    """Test editing file with end=None."""
    test_file = tmp_path / "none_end.txt"
    test_file.write_text("line1\nline2\nline3\n")

    # Get file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Test replacement with None as end
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 2,
                "end": 3,  # Replace lines 2 and 3
                "contents": "new2\nnew3\n",
                "range_hash": editor.calculate_hash("line2\nline3\n"),
            }
        ],
    )
    assert result["result"] == "ok"
    assert test_file.read_text() == "line1\nnew2\nnew3\n"


@pytest.mark.asyncio
async def test_edit_file_with_exceeding_line_end(editor, tmp_path):
    """Test editing file with end exceeding file length."""
    test_file = tmp_path / "exceed_end.txt"
    test_file.write_text("line1\nline2\nline3\n")

    # Get file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Test replacement with end > file length
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 2,
                "end": 10,  # File only has 3 lines
                "contents": "new2\nnew3\n",
                "range_hash": editor.calculate_hash("line2\nline3\n"),
            }
        ],
    )

    assert result["result"] == "ok"
    assert test_file.read_text() == "line1\nnew2\nnew3\n"

    # Clean up
    test_file.chmod(0o644)  # Restore write permission for cleanup


@pytest.mark.asyncio
async def test_new_file_with_non_empty_hash(editor, tmp_path):
    """Test handling of new file creation with non-empty hash."""
    new_file = tmp_path / "nonexistent.txt"
    result = await editor.edit_file_contents(
        str(new_file),
        "non_empty_hash",  # Non-empty hash for non-existent file
        [{"start": 1, "contents": "test content\n", "range_hash": ""}],
    )

    # Verify proper error handling
    assert result["result"] == "error"
    assert result["file_hash"] is None
    assert "content" not in result


@pytest.mark.asyncio
async def test_read_file_contents_with_start_beyond_total(editor, tmp_path):
    """Test read_file_contents when start is beyond total lines."""
    # Create a test file
    test_file = tmp_path / "test.txt"
    test_file.write_text("line1\nline2\nline3\n")

    # Call read_file_contents with start beyond total lines
    content, start, end, content_hash, total_lines, content_size = (
        await editor.read_file_contents(str(test_file), start=10)
    )

    # Verify empty content is returned
    assert content == ""
    assert start == 9  # start is converted to 0-based indexing
    assert end == 9
    assert content_hash == editor.calculate_hash("")
    assert total_lines == 3
    assert content_size == 0


@pytest.mark.asyncio
async def test_create_file_directory_creation_failure(editor, tmp_path, monkeypatch):
    """Test handling of directory creation failure when creating a new file."""
    # Create a path with multiple nested directories
    deep_path = tmp_path / "deeply" / "nested" / "path" / "test.txt"

    # Mock os.makedirs to raise an OSError
    def mock_makedirs(*args, **kwargs):
        raise OSError("Permission denied")

    monkeypatch.setattr("os.makedirs", mock_makedirs)

    # Attempt to create a new file
    result = await editor.edit_file_contents(
        str(deep_path),
        "",  # Empty hash for new file
        [
            {
                "line_start": 1,
                "contents": "test content\n",
            }
        ],
    )

    # Verify proper error handling
    assert result["result"] == "error"
    assert "Failed to create directory" in result["reason"]
    assert "Permission denied" in result["reason"]
    assert result["file_hash"] is None
    assert "content" not in result


@pytest.mark.asyncio
async def test_io_error_handling(editor, tmp_path, monkeypatch):
    """Test handling of IO errors during file operations."""
    test_file = tmp_path / "test.txt"
    content = "test content\n"
    test_file.write_text(content)

    def mock_open(*args, **kwargs):
        raise IOError("Test IO Error")

    monkeypatch.setattr("builtins.open", mock_open)

    result = await editor.edit_file_contents(
        str(test_file),
        "",
        [{"line_start": 1, "contents": "new content\n"}],
    )

    assert result["result"] == "error"
    assert "Error editing file" in result["reason"]
    assert "Test IO Error" in result["reason"]


@pytest.mark.asyncio
async def test_exception_handling(editor, tmp_path, monkeypatch):
    """Test handling of unexpected exceptions during file operations."""
    test_file = tmp_path / "test.txt"

    def mock_open(*args, **kwargs):
        raise Exception("Unexpected error")

    monkeypatch.setattr("builtins.open", mock_open)

    result = await editor.edit_file_contents(
        str(test_file),
        "",
        [{"line_start": 1, "contents": "new content\n"}],
    )

    assert result["result"] == "error"


@pytest.mark.asyncio
async def test_insert_operation(editor, tmp_path):
    """Test file insertion operations."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("line1\nline2\nline3\n")

    # Get file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Test insertion operation (inserting at line 2)
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 2,
                "end": None,  # For insertion mode (empty range_hash), end is optional
                "contents": "new line\n",
                "range_hash": "",  # Empty range_hash means insertion mode
            }
        ],
    )

    assert result["result"] == "ok"
    assert test_file.read_text() == "line1\nnew line\nline2\nline3\n"


@pytest.mark.asyncio
async def test_content_without_newline(editor, tmp_path):
    """Test handling content without trailing newline."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("line1\nline2\nline3\n")

    # Get file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Update with content that doesn't have a trailing newline
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 2,
                "end": 2,
                "contents": "new line",  # No trailing newline
                "range_hash": editor.calculate_hash("line2\n"),
            }
        ],
    )

    assert result["result"] == "ok"
    assert test_file.read_text() == "line1\nnew line\nline3\n"
    result = await editor.edit_file_contents(
        str(test_file),
        "",
        [{"start": 1, "contents": "new content\n"}],
    )

    assert result["result"] == "error"
    assert "Unexpected error" in result["reason"]


@pytest.mark.asyncio
async def test_invalid_line_range(editor, tmp_path):
    """Test handling of invalid line range where end line is less than start line."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("line1\nline2\nline3\n")

    # Try to read with invalid line range
    with pytest.raises(ValueError) as excinfo:
        await editor.read_file_contents(str(test_file), start=3, end=2)

    assert "End line must be greater than or equal to start line" in str(excinfo.value)

    # Try to edit with invalid line range
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 3,
                "end": 2,
                "contents": "new content\n",
                "range_hash": editor.calculate_hash("line3\n"),
            }
        ],
    )

    assert result["result"] == "error"
    assert "End line must be greater than or equal to start line" in result["reason"]


@pytest.mark.asyncio
async def test_append_mode(editor, tmp_path):
    """Test appending content when start exceeds total lines."""
    # Create a test file
    test_file = tmp_path / "test_append.txt"
    original_content = "Line 1\nLine 2\nLine 3\n"
    test_file.write_text(original_content)

    # Read the content and get hash
    content, start, end, file_hash, total_lines, size = await editor.read_file_contents(
        str(test_file)
    )

    # Attempt to append content with start > total_lines
    append_content = "Appended Line\n"
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": total_lines + 1,  # Start beyond current line count
                "contents": append_content,
                "range_hash": "",  # Empty range_hash for append mode
            }
        ],
    )

    assert result["result"] == "ok"
    assert test_file.read_text() == original_content + append_content


@pytest.mark.asyncio
async def test_dict_patch_with_defaults(editor: TextEditor, tmp_path):
    """Test dictionary patch with default values."""
    test_file = tmp_path / "test.txt"
    original_content = "line1\nline2\nline3\n"
    test_file.write_text(original_content)

    # Get first line content and calculate hashes
    first_line_content, _, _, _, _, _ = await editor.read_file_contents(
        str(test_file), start=1, end=1
    )
    file_hash = editor.calculate_hash(original_content)

    # Edit using dict patch with missing optional fields
    patch = {
        "contents": "new line\n",  # Add newline to maintain file structure
        "start": 1,
        "end": 1,  # Explicitly specify end
        "range_hash": editor.calculate_hash(first_line_content),
    }
    result = await editor.edit_file_contents(str(test_file), file_hash, [patch])

    assert result["result"] == "ok"
    # Should replace line 1 when range_hash is provided
    assert test_file.read_text() == "new line\nline2\nline3\n"


@pytest.mark.asyncio
async def test_edit_file_without_end(editor, tmp_path):
    """Test editing a file using dictionary patch without end."""
    test_file = tmp_path / "test.txt"
    content = "line1\nline2\nline3\n"
    test_file.write_text(content)

    # Create a patch with minimal fields
    patch = EditPatch(
        contents="new line\n",
        start=1,
        end=1,
        range_hash=editor.calculate_hash("line1\n"),
    )

    # Calculate file hash from original content
    file_hash = editor.calculate_hash(content)

    result = await editor.edit_file_contents(str(test_file), file_hash, [patch])

    assert result["result"] == "ok"
    assert test_file.read_text() == "new line\nline2\nline3\n"


def test_validate_environment():
    """Test environment validation."""
    # Currently _validate_environment is a placeholder
    # This test ensures the method exists and can be called without errors
    TextEditor()._validate_environment()


@pytest.mark.asyncio
async def test_io_error_during_final_write(editor, tmp_path, monkeypatch):
    """Test handling of IO errors during final content writing."""
    test_file = tmp_path / "test.txt"
    test_file.write_text("original content\n")

    # Get file hash
    content, _, _, file_hash, _, _ = await editor.read_file_contents(str(test_file))

    # Mock open to raise IOError during final write
    original_open = open
    open_count = 0

    def mock_open(*args, **kwargs):
        nonlocal open_count
        open_count += 1
        if open_count > 1:  # Allow first open for reading, fail on write
            raise IOError("Failed to write file")
        return original_open(*args, **kwargs)

    monkeypatch.setattr("builtins.open", mock_open)

    # Try to edit file with mocked write error
    result = await editor.edit_file_contents(
        str(test_file),
        file_hash,
        [
            {
                "start": 1,
                "end": 1,
                "contents": "new content\n",
                "range_hash": editor.calculate_hash("original content\n"),
            }
        ],
    )

    assert result["result"] == "error"
    assert "Error editing file" in result["reason"]
    assert "Failed to write file" in result["reason"]
    assert test_file.read_text() == "original content\n"  # File should be unchanged
    editor._validate_environment()


@pytest.mark.asyncio
async def test_initialization_with_environment_error(monkeypatch):
    """Test TextEditor initialization when environment validation fails."""

    def mock_validate_environment(self):
        raise EnvironmentError("Failed to validate environment")

    # Patch the _validate_environment method
    monkeypatch.setattr(TextEditor, "_validate_environment", mock_validate_environment)

    # Verify that initialization fails with the expected error
    with pytest.raises(EnvironmentError, match="Failed to validate environment"):
        TextEditor()


@pytest.mark.asyncio
async def test_read_file_not_found_error(editor, tmp_path):
    """Test FileNotFoundError handling when reading a non-existent file."""
    non_existent_file = tmp_path / "does_not_exist.txt"

    with pytest.raises(FileNotFoundError) as excinfo:
        await editor._read_file(str(non_existent_file))

    assert "File not found:" in str(excinfo.value)
    assert str(non_existent_file) in str(excinfo.value)



"""Handler for patching text file contents."""

import json
import logging
import os
import traceback
from typing import Any, Dict, Sequence

from mcp.types import TextContent, Tool

from .base import BaseHandler

logger = logging.getLogger("mcp-text-editor")


class PatchTextFileContentsHandler(BaseHandler):
    """Handler for patching a text file."""

    name = "patch_text_file_contents"
    description = "Apply patches to text files with hash-based validation for concurrency control.you need to use get_text_file_contents tool to get the file hash and range hash every time before using this tool. you can use append_text_file_contents tool to append text contents to the file without range hash, start and end. you can use insert_text_file_contents tool to insert text contents to the file without range hash, start and end."

    def get_tool_description(self) -> Tool:
        """Get the tool description."""
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "file_path": {
                        "type": "string",
                        "description": "Path to the text file. File path must be absolute.",
                    },
                    "file_hash": {
                        "type": "string",
                        "description": "Hash of the file contents for concurrency control.",
                    },
                    "patches": {
                        "type": "array",
                        "description": "List of patches to apply",
                        "items": {
                            "type": "object",
                            "properties": {
                                "start": {
                                    "type": "integer",
                                    "description": "Starting line number (1-based).it should match the range hash.",
                                },
                                "end": {
                                    "type": "integer",
                                    "description": "Ending line number (null for end of file).it should match the range hash.",
                                },
                                "contents": {
                                    "type": "string",
                                    "description": "New content to replace the range with",
                                },
                                "range_hash": {
                                    "type": "string",
                                    "description": "Hash of the content being replaced. it should get from get_text_file_contents tool with the same start and end.",
                                },
                            },
                            "required": ["start", "end", "contents", "range_hash"],
                        },
                    },
                    "encoding": {
                        "type": "string",
                        "description": "Text encoding (default: 'utf-8')",
                        "default": "utf-8",
                    },
                },
                "required": ["file_path", "file_hash", "patches"],
            },
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        """Execute the tool with given arguments."""
        try:
            if "file_path" not in arguments:
                raise RuntimeError("Missing required argument: file_path")
            if "file_hash" not in arguments:
                raise RuntimeError("Missing required argument: file_hash")
            if "patches" not in arguments:
                raise RuntimeError("Missing required argument: patches")

            file_path = arguments["file_path"]
            if not os.path.isabs(file_path):
                raise RuntimeError(f"File path must be absolute: {file_path}")

            # Check if file exists
            if not os.path.exists(file_path):
                raise RuntimeError(f"File does not exist: {file_path}")

            encoding = arguments.get("encoding", "utf-8")

            # Apply patches using editor.edit_file_contents
            result = await self.editor.edit_file_contents(
                file_path=file_path,
                expected_file_hash=arguments["file_hash"],
                patches=arguments["patches"],
                encoding=encoding,
            )

            return [TextContent(type="text", text=json.dumps(result, indent=2))]

        except Exception as e:
            logger.error(f"Error processing request: {str(e)}")
            logger.error(traceback.format_exc())
            raise RuntimeError(f"Error processing request: {str(e)}") from e



"""Test cases for append_text_file_contents handler."""

import os
from typing import Any, Dict, Generator

import pytest

from mcp_text_editor.server import AppendTextFileContentsHandler
from mcp_text_editor.text_editor import TextEditor

# Initialize handler for tests
append_handler = AppendTextFileContentsHandler()


@pytest.fixture
def test_dir(tmp_path: str) -> str:
    """Create a temporary directory for test files."""
    return str(tmp_path)


@pytest.fixture
def cleanup_files() -> Generator[None, None, None]:
    """Clean up any test files after each test."""
    yield
    # Add cleanup code if needed


@pytest.mark.asyncio
async def test_append_text_file_success(test_dir: str, cleanup_files: None) -> None:
    """Test successful appending to a file."""
    test_file = os.path.join(test_dir, "append_test.txt")
    initial_content = "Initial content\n"
    append_content = "Appended content\n"

    # Create initial file
    with open(test_file, "w", encoding="utf-8") as f:
        f.write(initial_content)

    # Get file hash for append operation
    editor = TextEditor()
    _, _, _, file_hash, _, _ = await editor.read_file_contents(test_file)

    # Append content using handler
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": append_content,
        "file_hash": file_hash,
    }
    response = await append_handler.run_tool(arguments)

    # Check if content was appended correctly
    with open(test_file, "r", encoding="utf-8") as f:
        content = f.read()
        assert content == initial_content + append_content

    # Parse response to check success
    assert len(response) == 1
    result = response[0].text
    assert '"result": "ok"' in result


@pytest.mark.asyncio
async def test_append_text_file_not_exists(test_dir: str, cleanup_files: None) -> None:
    """Test attempting to append to a non-existent file."""
    test_file = os.path.join(test_dir, "nonexistent.txt")

    # Try to append to non-existent file
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": "Some content\n",
        "file_hash": "dummy_hash",
    }

    # Should raise error because file doesn't exist
    with pytest.raises(RuntimeError) as exc_info:
        await append_handler.run_tool(arguments)
    assert "File does not exist" in str(exc_info.value)


@pytest.mark.asyncio
async def test_append_text_file_hash_mismatch(
    test_dir: str, cleanup_files: None
) -> None:
    """Test appending with incorrect file hash."""
    test_file = os.path.join(test_dir, "hash_test.txt")
    initial_content = "Initial content\n"

    # Create initial file
    with open(test_file, "w", encoding="utf-8") as f:
        f.write(initial_content)

    # Try to append with incorrect hash
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": "New content\n",
        "file_hash": "incorrect_hash",
    }

    # Should raise error because hash doesn't match
    with pytest.raises(RuntimeError) as exc_info:
        await append_handler.run_tool(arguments)
    assert "hash mismatch" in str(exc_info.value).lower()


@pytest.mark.asyncio
async def test_append_text_file_relative_path(
    test_dir: str, cleanup_files: None
) -> None:
    """Test attempting to append using a relative path."""
    arguments: Dict[str, Any] = {
        "file_path": "relative_path.txt",
        "contents": "Some content\n",
        "file_hash": "dummy_hash",
    }

    # Should raise error because path is not absolute
    with pytest.raises(RuntimeError) as exc_info:
        await append_handler.run_tool(arguments)
    assert "File path must be absolute" in str(exc_info.value)


@pytest.mark.asyncio
async def test_append_text_file_missing_args() -> None:
    """Test appending with missing arguments."""
    # Test missing path
    with pytest.raises(RuntimeError) as exc_info:
        await append_handler.run_tool({"contents": "content\n", "file_hash": "hash"})
    assert "Missing required argument: file_path" in str(exc_info.value)

    # Test missing contents
    with pytest.raises(RuntimeError) as exc_info:
        await append_handler.run_tool(
            {"file_path": "/absolute/path.txt", "file_hash": "hash"}
        )
    assert "Missing required argument: contents" in str(exc_info.value)

    # Test missing file_hash
    with pytest.raises(RuntimeError) as exc_info:
        await append_handler.run_tool(
            {"file_path": "/absolute/path.txt", "contents": "content\n"}
        )
    assert "Missing required argument: file_hash" in str(exc_info.value)


@pytest.mark.asyncio
async def test_append_text_file_custom_encoding(
    test_dir: str, cleanup_files: None
) -> None:
    """Test appending with custom encoding."""
    test_file = os.path.join(test_dir, "encode_test.txt")
    initial_content = "こんにちは\n"
    append_content = "さようなら\n"

    # Create initial file
    with open(test_file, "w", encoding="utf-8") as f:
        f.write(initial_content)

    # Get file hash for append operation
    editor = TextEditor()
    _, _, _, file_hash, _, _ = await editor.read_file_contents(
        test_file, encoding="utf-8"
    )

    # Append content using handler with specified encoding
    arguments: Dict[str, Any] = {
        "file_path": test_file,
        "contents": append_content,
        "file_hash": file_hash,
        "encoding": "utf-8",
    }
    response = await append_handler.run_tool(arguments)

    # Check if content was appended correctly
    with open(test_file, "r", encoding="utf-8") as f:
        content = f.read()
        assert content == initial_content + append_content

    # Parse response to check success
    assert len(response) == 1
    result = response[0].text
    assert '"result": "ok"' in result



"""Tests for core service logic."""

import os

import pytest

from mcp_text_editor.models import EditFileOperation, EditPatch, EditResult
from mcp_text_editor.service import TextEditorService


@pytest.fixture
def service():
    """Create TextEditorService instance."""
    return TextEditorService()


def test_calculate_hash(service):
    """Test hash calculation."""
    content = "test content"
    hash1 = service.calculate_hash(content)
    hash2 = service.calculate_hash(content)
    assert hash1 == hash2
    assert isinstance(hash1, str)
    assert len(hash1) == 64  # SHA-256 hash length


def test_read_file_contents(service, test_file):
    """Test reading file contents."""
    # Test reading entire file
    content, start, end = service.read_file_contents(test_file)
    assert content == "Line 1\nLine 2\nLine 3\nLine 4\nLine 5\n"
    assert start == 1
    assert end == 5

    # Test reading specific lines
    content, start, end = service.read_file_contents(test_file, start=2, end=4)
    assert content == "Line 2\nLine 3\nLine 4\n"
    assert start == 2
    assert end == 4


def test_read_file_contents_invalid_file(service):
    """Test reading non-existent file."""
    with pytest.raises(FileNotFoundError):
        service.read_file_contents("nonexistent.txt")


def test_validate_patches(service):
    """Test patch validation."""
    # Valid patches
    patches = [
        EditPatch(start=1, end=2, contents="content1", range_hash="hash1"),
        EditPatch(start=3, end=4, contents="content2", range_hash="hash2"),
    ]
    assert service.validate_patches(patches, 5) is True

    # Overlapping patches
    patches = [
        EditPatch(start=1, end=3, contents="content1", range_hash="hash1"),
        EditPatch(start=2, end=4, contents="content2", range_hash="hash2"),
    ]
    assert service.validate_patches(patches, 5) is False

    # Out of bounds patches
    patches = [EditPatch(start=1, end=10, contents="content1", range_hash="hash1")]
    assert service.validate_patches(patches, 5) is False


def test_edit_file_contents(service, tmp_path):
    """Test editing file contents."""
    # Create test file
    test_file = tmp_path / "edit_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Calculate initial hash
    initial_hash = service.calculate_hash(test_content)

    # Create edit operation
    operation = EditFileOperation(
        path=file_path,
        hash=initial_hash,
        patches=[EditPatch(start=2, end=2, contents="new line2", range_hash="hash1")],
    )

    # Apply edit
    result = service.edit_file_contents(file_path, operation)
    assert file_path in result
    edit_result = result[file_path]
    assert isinstance(edit_result, EditResult)
    assert edit_result.result == "ok"

    # Verify changes
    new_content = test_file.read_text()
    assert "new line2" in new_content


def test_edit_file_contents_hash_mismatch(service, tmp_path):
    """Test editing with hash mismatch."""
    # Create test file
    test_file = tmp_path / "hash_mismatch_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Create edit operation with incorrect hash
    operation = EditFileOperation(
        path=file_path,
        hash="incorrect_hash",
        patches=[EditPatch(start=2, end=2, contents="new line2", range_hash="hash1")],
    )

    # Attempt edit
    result = service.edit_file_contents(file_path, operation)
    assert file_path in result
    edit_result = result[file_path]
    assert edit_result.result == "error"
    assert "hash mismatch" in edit_result.reason.lower()


def test_edit_file_contents_invalid_patches(service, tmp_path):
    """Test editing with invalid patches."""
    # Create test file
    test_file = tmp_path / "invalid_patches_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Calculate initial hash
    initial_hash = service.calculate_hash(test_content)

    # Create edit operation with invalid patches
    operation = EditFileOperation(
        path=file_path,
        hash=initial_hash,
        patches=[
            EditPatch(
                start=1, end=10, contents="new content", range_hash="hash1"
            )  # Beyond file length
        ],
    )

    # Attempt edit
    result = service.edit_file_contents(file_path, operation)
    assert file_path in result
    edit_result = result[file_path]
    assert edit_result.result == "error"
    assert "invalid patch" in edit_result.reason.lower()


def test_edit_file_contents_file_error(service):
    """Test editing with file error."""
    file_path = "nonexistent.txt"
    # Attempt to edit non-existent file
    operation = EditFileOperation(
        path=file_path,
        hash="any_hash",
        patches=[EditPatch(contents="new content", range_hash="")],
    )

    # Attempt edit
    result = service.edit_file_contents(file_path, operation)
    assert file_path in result
    edit_result = result[file_path]
    assert edit_result.result == "error"
    assert "no such file" in edit_result.reason.lower()


def test_edit_file_unexpected_error(service, tmp_path):
    """Test handling of unexpected errors during file editing."""
    # Setup test file and operation
    test_file = str(tmp_path / "error_test.txt")
    operation = EditFileOperation(
        path=test_file,
        hash="dummy_hash",
        patches=[EditPatch(contents="test content\n", start=1, range_hash="")],
    )

    # Try to edit non-existent file
    result = service.edit_file_contents(test_file, operation)
    edit_result = result[test_file]

    # Verify error handling
    assert edit_result.result == "error"
    assert "no such file" in edit_result.reason.lower()
    assert edit_result.hash is None


def test_edit_file_contents_permission_error(service, tmp_path):
    """Test handling of permission errors during file editing."""
    # Create test file
    test_file = tmp_path / "general_error_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Make the file read-only to cause a permission error
    os.chmod(file_path, 0o444)

    # Create edit operation
    operation = EditFileOperation(
        path=file_path,
        hash=service.calculate_hash(test_content),
        patches=[EditPatch(start=2, end=2, contents="new line2", range_hash="hash1")],
    )

    # Attempt edit
    result = service.edit_file_contents(file_path, operation)
    edit_result = result[file_path]

    assert edit_result.result == "error"
    assert "permission denied" in edit_result.reason.lower()
    assert edit_result.hash is None

    # Clean up
    os.chmod(file_path, 0o644)


def test_edit_file_contents_general_exception(service, mocker):
    """Test handling of general exceptions during file editing."""
    test_file = "test.txt"
    operation = EditFileOperation(
        path=test_file,
        hash="hash123",
        patches=[EditPatch(contents="new content", start=1, range_hash="")],
    )

    # Mock edit_file to raise an exception
    # Create a test file
    with open(test_file, "w") as f:
        f.write("test content\n")

    try:
        # Mock os.path.exists to return True
        mocker.patch("os.path.exists", return_value=True)
        # Mock open to raise an exception
        mocker.patch(
            "builtins.open",
            side_effect=Exception("Unexpected error during file operation"),
        )

        result = service.edit_file_contents(test_file, operation)
        edit_result = result[test_file]

        assert edit_result.result == "error"
        assert "unexpected error" in edit_result.reason.lower()

    finally:
        # Clean up
        import os

        mocker.stopall()
        if os.path.exists(test_file):
            os.remove(test_file)
    assert edit_result.hash is None



"""Handler for inserting content into text files."""

import json
import logging
import os
import traceback
from typing import Any, Dict, Sequence

from mcp.types import TextContent, Tool

from .base import BaseHandler

logger = logging.getLogger("mcp-text-editor")


class InsertTextFileContentsHandler(BaseHandler):
    """Handler for inserting content before or after a specific line in a text file."""

    name = "insert_text_file_contents"
    description = "Insert content before or after a specific line in a text file. Uses hash-based validation for concurrency control. You need to provide the file_hash comes from get_text_file_contents."

    def get_tool_description(self) -> Tool:
        """Get the tool description."""
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "file_path": {
                        "type": "string",
                        "description": "Path to the text file. File path must be absolute.",
                    },
                    "file_hash": {
                        "type": "string",
                        "description": "Hash of the file contents for concurrency control. it should be matched with the file_hash when get_text_file_contents is called.",
                    },
                    "contents": {
                        "type": "string",
                        "description": "Content to insert",
                    },
                    "before": {
                        "type": "integer",
                        "description": "Line number before which to insert content (mutually exclusive with 'after')",
                    },
                    "after": {
                        "type": "integer",
                        "description": "Line number after which to insert content (mutually exclusive with 'before')",
                    },
                    "encoding": {
                        "type": "string",
                        "description": "Text encoding (default: 'utf-8')",
                        "default": "utf-8",
                    },
                },
                "required": ["file_path", "file_hash", "contents"],
            },
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        """Execute the tool with given arguments."""
        try:
            if "file_path" not in arguments:
                raise RuntimeError("Missing required argument: file_path")
            if "file_hash" not in arguments:
                raise RuntimeError("Missing required argument: file_hash")
            if "contents" not in arguments:
                raise RuntimeError("Missing required argument: contents")

            file_path = arguments["file_path"]
            if not os.path.isabs(file_path):
                raise RuntimeError(f"File path must be absolute: {file_path}")

            # Check if exactly one of before/after is specified
            if ("before" in arguments) == ("after" in arguments):
                raise RuntimeError(
                    "Exactly one of 'before' or 'after' must be specified"
                )

            line_number = (
                arguments.get("before")
                if "before" in arguments
                else arguments.get("after")
            )
            is_before = "before" in arguments
            encoding = arguments.get("encoding", "utf-8")

            # Get result from editor
            result = await self.editor.insert_text_file_contents(
                file_path=file_path,
                file_hash=arguments["file_hash"],
                contents=arguments["contents"],
                before=line_number if is_before else None,
                after=None if is_before else line_number,
                encoding=encoding,
            )
            # Wrap result with file_path key
            result = {file_path: result}
            return [TextContent(type="text", text=json.dumps(result, indent=2))]

        except Exception as e:
            logger.error(f"Error processing request: {str(e)}")
            logger.error(traceback.format_exc())
            raise RuntimeError(f"Error processing request: {str(e)}") from e



"""Tests for delete_text_file_contents functionality."""

import json

import pytest

from mcp_text_editor.models import DeleteTextFileContentsRequest, FileRange
from mcp_text_editor.service import TextEditorService


@pytest.fixture
def service():
    """Create TextEditorService instance."""
    return TextEditorService()


def test_delete_text_file_contents_basic(service, tmp_path):
    """Test basic delete operation."""
    # Create test file
    test_file = tmp_path / "delete_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Calculate initial hash
    initial_hash = service.calculate_hash(test_content)

    # Create delete request
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash=initial_hash,
        ranges=[
            FileRange(start=2, end=2, range_hash=service.calculate_hash("line2\n"))
        ],
        encoding="utf-8",
    )

    # Apply delete
    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "ok"

    # Verify changes
    new_content = test_file.read_text()
    assert new_content == "line1\nline3\n"


def test_delete_text_file_contents_hash_mismatch(service, tmp_path):
    """Test deleting with hash mismatch."""
    # Create test file
    test_file = tmp_path / "hash_mismatch_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Create delete request with incorrect hash
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash="incorrect_hash",
        ranges=[
            FileRange(start=2, end=2, range_hash=service.calculate_hash("line2\n"))
        ],
        encoding="utf-8",
    )

    # Attempt delete
    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "error"
    assert "hash mismatch" in delete_result.reason.lower()


def test_delete_text_file_contents_invalid_ranges(service, tmp_path):
    """Test deleting with invalid ranges."""
    # Create test file
    test_file = tmp_path / "invalid_ranges_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Calculate initial hash
    initial_hash = service.calculate_hash(test_content)

    # Create delete request with invalid ranges
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash=initial_hash,
        ranges=[FileRange(start=1, end=10, range_hash="hash1")],  # Beyond file length
        encoding="utf-8",
    )

    # Attempt delete
    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "error"
    assert "invalid ranges" in delete_result.reason.lower()


def test_delete_text_file_contents_range_hash_mismatch(service, tmp_path):
    """Test deleting with range hash mismatch."""
    # Create test file
    test_file = tmp_path / "range_hash_test.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Calculate initial hash
    initial_hash = service.calculate_hash(test_content)

    # Create delete request with incorrect range hash
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash=initial_hash,
        ranges=[FileRange(start=2, end=2, range_hash="incorrect_hash")],
        encoding="utf-8",
    )

    # Attempt delete
    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "error"
    assert "hash mismatch for range" in delete_result.reason.lower()


def test_delete_text_file_contents_relative_path(service, tmp_path):
    """Test deleting with a relative file path."""
    # Create delete request with relative path
    request = DeleteTextFileContentsRequest(
        file_path="relative/path.txt",
        file_hash="some_hash",
        ranges=[FileRange(start=1, end=1, range_hash="hash1")],
        encoding="utf-8",
    )

    # Attempt delete
    result = service.delete_text_file_contents(request)
    assert "relative/path.txt" in result
    delete_result = result["relative/path.txt"]
    assert delete_result.result == "error"
    assert "no such file or directory" in delete_result.reason.lower()


def test_delete_text_file_contents_empty_ranges(service, tmp_path):
    """Test deleting with empty ranges list."""
    test_file = tmp_path / "empty_ranges.txt"
    test_content = "line1\nline2\nline3\n"
    test_file.write_text(test_content)
    file_path = str(test_file)
    content_hash = service.calculate_hash(test_content)

    # Test empty ranges
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash=content_hash,
        ranges=[],  # Empty ranges list
        encoding="utf-8",
    )

    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "error"
    assert "missing required argument: ranges" in delete_result.reason.lower()


def test_delete_text_file_contents_nonexistent_file(service, tmp_path):
    """Test deleting content from a nonexistent file."""
    file_path = str(tmp_path / "nonexistent.txt")

    # Create delete request for nonexistent file
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash="some_hash",
        ranges=[FileRange(start=1, end=1, range_hash="hash1")],
        encoding="utf-8",
    )

    # Attempt delete
    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "error"
    assert "no such file or directory" in delete_result.reason.lower()


def test_delete_text_file_contents_multiple_ranges(service, tmp_path):
    """Test deleting multiple ranges simultaneously."""
    # Create test file
    test_file = tmp_path / "multiple_ranges_test.txt"
    test_content = "line1\nline2\nline3\nline4\nline5\n"
    test_file.write_text(test_content)
    file_path = str(test_file)

    # Calculate initial hash
    initial_hash = service.calculate_hash(test_content)

    # Create delete request with multiple ranges
    request = DeleteTextFileContentsRequest(
        file_path=file_path,
        file_hash=initial_hash,
        ranges=[
            FileRange(start=2, end=2, range_hash=service.calculate_hash("line2\n")),
            FileRange(start=4, end=4, range_hash=service.calculate_hash("line4\n")),
        ],
        encoding="utf-8",
    )

    # Apply delete
    result = service.delete_text_file_contents(request)
    assert file_path in result
    delete_result = result[file_path]
    assert delete_result.result == "ok"

    # Verify changes
    new_content = test_file.read_text()
    assert new_content == "line1\nline3\nline5\n"


@pytest.mark.asyncio
async def test_delete_text_file_contents_handler_validation():
    """Test validation in DeleteTextFileContentsHandler."""
    from mcp_text_editor.handlers.delete_text_file_contents import (
        DeleteTextFileContentsHandler,
    )
    from mcp_text_editor.text_editor import TextEditor

    editor = TextEditor()
    handler = DeleteTextFileContentsHandler(editor)

    # Test missing file_hash
    with pytest.raises(RuntimeError) as exc_info:
        arguments = {
            "file_path": "/absolute/path.txt",
            "ranges": [{"start": 1, "end": 1, "range_hash": "hash1"}],
            "encoding": "utf-8",
        }
        await handler.run_tool(arguments)
    assert "Missing required argument: file_hash" in str(exc_info.value)

    # Test missing ranges
    with pytest.raises(RuntimeError) as exc_info:
        arguments = {
            "file_path": "/absolute/path.txt",
            "file_hash": "some_hash",
            "encoding": "utf-8",
        }
        await handler.run_tool(arguments)
    assert "Missing required argument: ranges" in str(exc_info.value)

    # Test missing file_path
    with pytest.raises(RuntimeError) as exc_info:
        arguments = {
            "file_hash": "some_hash",
            "ranges": [{"start": 1, "end": 1, "range_hash": "hash1"}],
            "encoding": "utf-8",
        }
        await handler.run_tool(arguments)
    assert "Missing required argument: file_path" in str(exc_info.value)

    # Test relative file path
    with pytest.raises(RuntimeError) as exc_info:
        arguments = {
            "file_path": "relative/path.txt",
            "file_hash": "some_hash",
            "ranges": [{"start": 1, "end": 1, "range_hash": "hash1"}],
            "encoding": "utf-8",
        }
        await handler.run_tool(arguments)
    assert "File path must be absolute" in str(exc_info.value)


@pytest.mark.asyncio
async def test_delete_text_file_contents_handler_runtime_error(tmp_path):
    """Test runtime error handling in DeleteTextFileContentsHandler."""
    from mcp_text_editor.handlers.delete_text_file_contents import (
        DeleteTextFileContentsHandler,
    )
    from mcp_text_editor.service import TextEditorService
    from mcp_text_editor.text_editor import TextEditor

    class MockService(TextEditorService):
        def delete_text_file_contents(self, request):
            raise RuntimeError("Mock error during delete")

    editor = TextEditor()
    editor.service = MockService()
    handler = DeleteTextFileContentsHandler(editor)

    test_file = tmp_path / "error_test.txt"
    test_file.write_text("test content")

    with pytest.raises(RuntimeError) as exc_info:
        arguments = {
            "file_path": str(test_file),
            "file_hash": "some_hash",
            "ranges": [{"start": 1, "end": 1, "range_hash": "hash1"}],
            "encoding": "utf-8",
        }
        await handler.run_tool(arguments)

    assert "Error processing request: Mock error during delete" in str(exc_info.value)


@pytest.mark.asyncio
async def test_delete_text_file_contents_handler_success(tmp_path):
    """Test successful execution of DeleteTextFileContentsHandler including JSON serialization."""
    from mcp_text_editor.handlers.delete_text_file_contents import (
        DeleteTextFileContentsHandler,
    )
    from mcp_text_editor.models import EditResult
    from mcp_text_editor.service import TextEditorService
    from mcp_text_editor.text_editor import TextEditor

    class MockService(TextEditorService):
        def delete_text_file_contents(self, request):
            return {
                request.file_path: EditResult(result="ok", hash="new_hash", reason=None)
            }

    editor = TextEditor()
    editor.service = MockService()
    handler = DeleteTextFileContentsHandler(editor)

    test_file = tmp_path / "test.txt"
    test_file.write_text("test content")

    arguments = {
        "file_path": str(test_file),
        "file_hash": "some_hash",
        "ranges": [{"start": 1, "end": 1, "range_hash": "hash1"}],
    }

    result = await handler.run_tool(arguments)
    assert len(result) == 1
    assert result[0].type == "text"

    # Check if response is JSON serializable
    response = json.loads(result[0].text)
    assert str(test_file) in response
    assert response[str(test_file)]["result"] == "ok"
    assert response[str(test_file)]["hash"] == "new_hash"



"""Test module for patch_text_file with end=None."""

import os

import pytest

from mcp_text_editor.text_editor import TextEditor


@pytest.mark.asyncio
async def test_patch_text_file_end_none(tmp_path):
    """Test patching text file with end=None."""
    # Create a test file
    file_path = os.path.join(tmp_path, "test.txt")
    editor = TextEditor()

    # Create initial content
    content = "line1\nline2\nline3\nline4\nline5\n"
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

    # Get file hash and range hash
    file_info = await editor.read_multiple_ranges(
        [
            {
                "file_path": str(file_path),
                "ranges": [{"start": 2, "end": None}],  # Test with end=None
            }
        ]
    )

    # Extract file and range hashes
    file_content = file_info[str(file_path)]
    file_hash = file_content["file_hash"]
    range_hash = file_content["ranges"][0]["range_hash"]

    # Patch the file
    new_content = "new line2\nnew line3\nnew line4\nnew line5\n"
    patch = {
        "start": 2,
        "end": None,  # Test with end=None
        "contents": new_content,
        "range_hash": range_hash,
    }
    result = await editor.edit_file_contents(
        str(file_path),
        file_hash,
        [patch],
    )

    # Verify the patch was successful
    assert result["result"] == "ok"
    with open(file_path, "r", encoding="utf-8") as f:
        updated_content = f.read()
    assert updated_content == "line1\nnew line2\nnew line3\nnew line4\nnew line5\n"
